package EppClient::Controller::Changepw;

use strict;
use warnings;
use base qw(Catalyst::Controller::RequestToken);

=head1 NAME

EppClient::Controller::Changepw - Catalyst Controller

=head1 DESCRIPTION

Catalyst Controller.

=head1 METHODS

=cut


=head2 index
    
Change password logic
    
=cut
    
sub index : Private {
    my ($self, $c) = @_;

    $c->log->info("EppClient: Changepw: changepw: present change pw page");

    # Recreate token when menu is pressed, this will cleanup any errors
    $c->session->{'pw_token'} = $self->create_token($c);

    $c->stash->{template} = 'changepw.tt2';
}

sub changepw_do : Local {
    my ($self, $c) = @_;

    $c->stash->{cmd_show_status} = 1;
    $c->stash->{cmd_success}     = 0;

    $c->forward('validate_and_create_csrf_token');

    # Get the registrar and passwords from form
    my $clid   = $c->req->param("clid");
    my $pw     = $c->req->param("curpw");
    my $newpw  = $c->req->param("newpw");
    my $rnewpw = $c->req->param("rnewpw");

    $c->log->info("EppClient: Changepw: changepw_do: registrar: '$clid'");

    # Verify that clid and password belongs to this session
    if ($c->session && $clid && $pw && $clid eq $c->session->{clid} && $newpw && $rnewpw) {

	if ($pw ne $c->session->{pw}) {
	    push @{$c->stash->{opsummary}},  "Password error";
	} elsif ($newpw ne $rnewpw) {
	    push @{$c->stash->{opsummary}},  "The new passwords do not match.";
	} else {
	    # Log the user in and set the new EPP password as well
	    $c->stash->{clid}  = $clid;
	    $c->stash->{pw}    = $pw;
	    $c->stash->{newpw} = $newpw;
	    $c->log->info("EppClient: Login: index: Attempt to log the user in");

	    my ($resok, $dri, $rc, $s) = EppClient::Model::NORID::DRI::dri_login_ok($c, 'string');

	    if ($resok) {
		# Successful, so let them use the application
		$c->log->info('EppClient: Changepw: change password was a SUCCESS!!');
		push @{$c->stash->{opsummary}},  "Password has been changed.";
		$c->stash->{pw}   = $newpw;
		$c->session->{pw} = $newpw;
		$c->stash->{cmd_success} = 1;
		$c->detach('/menu');

	    } else {
		# some error, typically wrong password since we have the correct clid.
		push @{$c->stash->{opsummary}},  "Cannot set new password.";
		push @{$c->stash->{opsummary}},  "Passwords must be 8-16 characters.";
		push @{$c->stash->{login_error_details}}, split "\n", $s if ($s);
	    } 
 	}
    } else {
	push @{$c->stash->{opsummary}},  "Too few parameters, all must be specified.";
    }
    $c->log->info('EppClient: change password failed!!');
    # If either of above don't work out, send back to the changepw page
    $c->stash->{template} = 'changepw.tt2';
}


=head2 access_denied

This is effectively the acl_access_denied method used by the ACL-plugin, see lib/EppClient.pm.

=cut

sub access_denied : Private {
    my ( $self, $c, $action ) = @_;

    # attempt to run changepw command, which is disabled.
    # Just do a quiet retreat back to the /menu
    $c->detach("/menu");
}

=head2 validate_and_create_csrf_token

CSRF validation.

=cut

sub validate_and_create_csrf_token: Private {
    my ( $self, $c) = @_;

    &EppClient::Controller::Util::validate_and_create_csrf_token($self, $c, 'pw');

}


=head1 AUTHOR

Trond Haugen

=head1 LICENSE

This library is free software, you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

1;
