#!/usr/bin/perl
use strict;
use warnings;
use Test::More tests => 31;
use Test::WWW::Mechanize::Catalyst 'EppClient';
use EppClient::InitTests;

my $tc = EppClient::InitTests::get_test_config;
my $t = $tc->{t};

my $rega = $t->{rega};
my $pwa  = $t->{pwa};

my $ua1 = Test::WWW::Mechanize::Catalyst->new;

$ua1->get_ok("http://localhost/", 'get localhost OK');

is($ua1->ct, "text/html", 'text/html OK');

$ua1->title_is("Norid EPP Client", "On the EPP Client's root page");

$ua1->content_contains("You need to log in to use this application", "Yup, on the login page");

### login without anything
$ua1->get_ok("http://localhost/login", "Login without anything shall fail");
$ua1->content_contains("You need to log in to use this application", "[023: Q-1] Failed OK");

### login without registrar
$ua1->get_ok("http://localhost/login?command=Login&clid=&pw=$pwa", "Login without registrar shall fail");
$ua1->content_contains("Missing mandatory parameter 'Registrar'", "Says no registrar");
$ua1->content_contains("You need to log in to use this application", "[023: Q-1]  Failed OK");

### login without pw
$ua1->get_ok("http://localhost/login?command=Login&clid=$rega", "Login without pw shall fail");
$ua1->content_contains("You need to log in to use this application","[023: Q-1] Failed OK");
$ua1->content_contains("Missing mandatory parameter 'Password'", "Says no password");

### login with correct pw
$ua1->get_ok("http://localhost/login?command=Login&clid=$rega&pw=$pwa", "Login OK");
is($ua1->ct, "text/html", 'text/html OK');
$ua1->title_is("Norid EPP Client", "Inside the EPP Client.");
$ua1->content_contains("Main menu", "[023-H-1] [023-H-2] [023-Q-1] Main menu found");
$ua1->content_contains("Please select one of the above tabs for a specific operation.", "[023-H-1] [023: Q-1] Login OK, main menu found, I'm sure!");

### Load the Change password page
$ua1->get_ok("http://localhost/changepw", "Get the password page");
$ua1->content_contains("Change password", "[023-H-3] [023: Q-3] Loaded the Change password page OK");
my $curclid = $ua1->form_number('1')->value('curclid');

### Try with a wrong cur password
my $curpw = $pwa . "X";
my $newpw = $pwa . "abc1";
my $rnewpw = $newpw;
$ua1->get_ok("http://localhost/changepw/changepw_do?submit=Change&curclid=$curclid&clid=$curclid&curpw=$curpw&newpw=$newpw&rnewpw=$rnewpw", "[023-Q-3] Change password fail");
$ua1->content_contains("Password error", "[023-H-3] [023: Q-3] Change password failed OK. Wrong current password");
#print $ua1->content;


### Try with a password too short
$curpw = $pwa;
$newpw = 'abc1';
$rnewpw = $newpw;
$ua1->get_ok("http://localhost/changepw/changepw_do?submit=Change&curclid=$curclid&clid=$curclid&curpw=$curpw&newpw=$newpw&rnewpw=$rnewpw", "[023-Q-3] Change password fail");
$ua1->content_contains("Passwords must be 8-16 characters", "[023-H-3] [023: Q-3] Change password failed OK. Too short password: $newpw");
#print $ua1->content;


### Try with a password that does not match
$curpw = $pwa;
$newpw = 'abc123';
$rnewpw = 'abc133';
$ua1->get_ok("http://localhost/changepw/changepw_do?submit=Change&curclid=$curclid&clid=$curclid&curpw=$curpw&newpw=$newpw&rnewpw=$rnewpw", "Change password fail");
$ua1->content_contains("The new passwords do not match", "[023-H-3] [023-Q-3] Change password failed OK. Not identical passwords: $newpw and $rnewpw");
#print $ua1->content;


### Fill in the form
$curpw = $pwa;
$newpw = $pwa . "again";
$rnewpw = $newpw;
$ua1->get_ok("http://localhost/changepw/changepw_do?submit=Change&curclid=$curclid&clid=$curclid&curpw=$curpw&newpw=$newpw&rnewpw=$rnewpw", "Change password");
$ua1->content_contains("Password has been changed", "[023-H-3] [023-Q-3] Changed password OK");


### Change back
$curpw = $pwa . "again";
$newpw = $pwa;
$rnewpw = $newpw;
$ua1->get_ok("http://localhost/changepw/changepw_do?submit=Change&curclid=$curclid&clid=$curclid&curpw=$curpw&newpw=$newpw&rnewpw=$rnewpw", "Change password");
$ua1->content_contains("Password has been changed", "[023-H-3] [023-Q-3] Changed password OK");

# Logout
$ua1->get_ok("http://localhost/logout", "Logout");
$ua1->content_contains("You need to log in to use this application", "[023: Q-6] Logout OK");
