#!/usr/bin/perl
use strict;
use warnings;
use Test::More "no_plan";
use Test::WWW::Mechanize::Catalyst 'EppClient';
use EppClient::Controller::Util;
use EppClient::InitTests;

my $tc = EppClient::InitTests::get_test_config;
my $t = $tc->{t};

my $rega = $t->{rega};
my $regb = $t->{regb};
my $regf = $t->{regf};
my $pwa  = $t->{pwa};
my $pwb  = $t->{pwb};
my $pwf  = $t->{pwf};

my $today  = EppClient::Controller::Util::timestamp_as_date(time);
my $future = EppClient::Controller::Util::timestamp_as_date(time + (60*60*48));
my $past   = EppClient::Controller::Util::timestamp_as_date(time - (60*60*48));

# string to make names unique, todays date + process id
my $sfx = "$today-$$";

my $ua1 = Test::WWW::Mechanize::Catalyst->new;
my $ua2 = Test::WWW::Mechanize::Catalyst->new;
my $uaf;
if ($regf) {
    $uaf = Test::WWW::Mechanize::Catalyst->new;
    is(1, 1, "Running with regf facets tests");
} else {
    is(1, 1, "Running without regf facets tests");
}

$ua1->get_ok("http://localhost/", 'get localhost OK');

is($ua1->ct, "text/html", 'text/html OK');

$ua1->title_is("Norid EPP Client", "On the EPP Client's root page");

$ua1->content_contains("You need to log in to use this application", "Yup, on the login page");

# login with correct pw
$ua1->get_ok("http://localhost/login?command=Login&clid=$rega&pw=$pwa", "Login $rega OK");
$ua2->get_ok("http://localhost/login?command=Login&clid=$regb&pw=$pwb", "Login $regb OK");

if ($uaf) {
    $uaf->get_ok("http://localhost/login?command=Login&clid=$regf&pw=$pwf", "Login $regf OK");
}

is($ua1->ct, "text/html", 'text/html OK');
$ua1->title_is("Norid EPP Client", "Inside the EPP Client");
$ua1->content_contains("Main menu", "Main menu found");
$ua1->content_contains("Please select one of the above tabs for a specific operation.", "Main menu found, I'm sure!");


############### Host tab ###############

# Create: A contact person contact with ok parameters
my $contactname="Carl Host Role Barks-$sfx";
my $type='role';
my $cc='NO';
my $pc='NO-7005';
my $city='Trondheim';
my $email='carl.host-barks@data.no';

$ua1->get_ok("http://localhost/contact/action?contactop=Create&contactname=$contactname&type=$type&cc=$cc&pc=$pc&city=$city&email=$email", "Create person contact");
my $srid_p = $ua1->form_number('1')->value('srid');
$ua1->content_contains("create '$contactname' succeeded", "Created person '$contactname' ($srid_p)");

# create a contact on $regb
$ua2->get_ok("http://localhost/contact/action?contactop=Create&contactname=$contactname&type=$type&cc=$cc&pc=$pc&city=$city&email=$email", "Create person contact");
my $srid_p2 = $ua2->form_number('1')->value('srid');
$ua2->content_contains("create '$contactname' succeeded", "Created person '$contactname' ($srid_p)");


####
# Create
#

# illegal name 1
my $hostname_1="ns1.host-ba_rks-$sfx.net";
$ua1->get_ok("http://localhost/host/action/?hostop=Create&hostname=$hostname_1", "Create host with illegal name: $hostname_1..." );
#print $ua1->content;
$ua1->content_contains("Illegal host name '$hostname_1'", "[023-J-2] Create host with illegal name: $hostname_1 was rejected, OK" );

# invalid name 2 : ACE-forms not allowed
$hostname_1 = 'ns1.xn--l-4ga.no';
$ua1->get_ok("http://localhost/host/action/?hostop=Create&hostname=$hostname_1", "Create host with illegal ACE name: $hostname_1..." );
$ua1->content_contains("create '$hostname_1' failed.", "[023-J-2] Create host with illegal ACE name: $hostname_1 was rejected" );

$ua1->content_contains("The hostname [$hostname_1] contains ACE encoded parts.", "[023-J-2] Create host with illegal ACE name: $hostname_1 was rejected" );

# name too short 
$hostname_1="x";
$ua1->get_ok("http://localhost/host/action/?hostop=Create&hostname=$hostname_1", "Create host with illegal name: $hostname_1..." );
$ua1->content_contains("create '$hostname_1' failed", "[023-J-2] Create host with illegal name: $hostname_1 was rejected, failed" );
$ua1->content_contains("Malformed hostname [$hostname_1]", "[023-J-2] Create host with illegal name: $hostname_1 was rejected, malformed" );

# Illegal ipv4
$hostname_1="ns1.host-$sfx.no";
my $v4_1='1.300.1.1';
$ua1->get_ok("http://localhost/host/action/?hostop=Create&hostname=$hostname_1&v46=$v4_1", "Create host with illegal IPV4-address: $v4_1 ..." );
$ua1->content_contains("Illegal IPV4-address '$v4_1'", "[023-J-2] Create host with illegal IPV4-address : $v4_1 was rejected, OK" );

# Illegal ipv6
$hostname_1="ns1.host-$sfx.no";
my $v6_1='2001:70000:60:215:f2ff:fe3e:fe65';
$ua1->get_ok("http://localhost/host/action/?hostop=Create&hostname=$hostname_1&v46=$v6_1", "Create host with illegal IPV6-address: $v6_1 ..." );
$ua1->content_contains("Illegal IPV6-address '$v6_1'", "[023-J-2] Create host with illegal IPV6-address : $v6_1 was rejected, OK" );

# External host name and ip is given, shall be rejected
$hostname_1="ns1.host-$sfx.net";
$v4_1='1.123.1.1';
$ua1->get_ok("http://localhost/host/action/?hostop=Create&hostname=$hostname_1&v46=$v4_1", "Create external host $hostname_1 with IPV4-address: $v4_1" );
#print $ua1->content;
$ua1->content_contains("create '$hostname_1' failed.", "[023-J-2] Create external host $hostname_1 with IP-address was rejected");
$ua1->content_contains("Hosts outside of the .no zone cannot be registered with IP addresses.", "[023-J-2] Create external host $hostname_1 with IP-address was rejected");

# .no host and ip is NOT given, shall be accepted
$hostname_1="ns1.host-$sfx.no";
$ua1->get_ok("http://localhost/host/action/?hostop=Create&hostname=$hostname_1", "Create .no host $hostname_1 without IP-address ..." );
$ua1->content_contains("create '$hostname_1' succeeded.", "[023-J-2] Create .no host $hostname_1 without IP-address was accepted" );


my $hn="ns1.host2-$sfx.no";
my $v4a='1.1.1.1';
my $v6a='2001:700:1:60:215:f2ff:fe3e:fe65';

# illegal contact ID
my $illc="ILL12";
$ua1->get_ok("http://localhost/host/action/?hostop=Create&hostname=$hn&v46=$v4a&v46=$v6a&contact=$illc", 'Create host with illegal contact ID ...');
$ua1->content_contains("Illegal Contact ID '$illc'", "[023-J-2] Create host with illegal contact ID failed");

# non-existing contact ID
my $nxc="ILLX12R";
$ua1->get_ok("http://localhost/host/action/?hostop=Create&hostname=$hn&v46=$v4a&v46=$v6a&contact=$nxc", 'Create host with non-existing contact ID ...');

#print $ua1->content;

$ua1->content_contains("create '$hn' failed.", "[023-J-2] Create host with non-existing contact ID failed");
$ua1->content_contains("Contact with handle [$nxc] not found.", "[023-J-2] Create host with non-existing contact ID failed");

#print $ua1->content;
# OK: Create a host

# OK .no host
$ua1->get_ok("http://localhost/host/action/?hostop=Create&hostname=$hn&v46=$v4a&v46=$v6a", 'Create host within .no ...');
$ua1->content_contains("create '$hn' succeeded", "[023-J-2] Create '$hn' within .no OK");

# OK .no host, all attributes, also contact
my $hn2="ns1.host-$sfx-2.no";
$ua1->get_ok("http://localhost/host/action/?hostop=Create&hostname=$hn2&v46=$v4a&v46=$v6a&contact=$srid_p", 'Create host within .no, with contact...');
$ua1->content_contains("create '$hn2' succeeded", "[023-J-2] Create '$hn2' within .no with contact");

# OK .no host, all attributes, also contact, for $regb also
my $hn3="ns1.host-$sfx-3.no";
$ua2->get_ok("http://localhost/host/action/?hostop=Create&hostname=$hn3&v46=$v4a&v46=$v6a&contact=$srid_p2", 'Create host within .no, with contact under $regb ...');
$ua2->content_contains("create '$hn3' succeeded", "[023-J-2] Create '$hn3' within .no with contact under $regb");

# external host
my $hnet="ns1.host-$sfx.net";
$ua1->get_ok("http://localhost/host/action/?hostop=Create&hostname=$hnet", 'Create host outside .no ');
$ua1->content_contains("create '$hnet' succeeded", "[023-J-2] Create host '$hnet' outside .no OK");


### 
# Operations Check', 'Info', 'Update', 'Delete':
#

my @ops = ('Check', 'Info', 'Update', 'Delete');

# Operations without host name
foreach my $op (@ops) {
    my $lop = lc($op);
    $ua1->get_ok("http://localhost/host/action?hostop=$op&hostname=&v46=1.2.3.4", "$op on missing host name");
    $ua1->content_contains("'$lop' requires a host name", "[023-I-5] $op on missing host name failed");
}

# invalid host names 1
foreach my $illname ("ns1.ille_gal-$sfx.no") {
    foreach my $op (@ops) {
	my $lop = lc($op);
	$ua1->get_ok("http://localhost/host/action?hostop=$op&hostname=$illname&v46=1.2.3.4", "$op on invalid host name");
	$ua1->content_contains("Illegal host name '$illname'", "[023-I-5] $op on invalid host name failed");
    }
}



# nonexisting name
my $nxname = "ns1.nonexisting-$sfx-nxt.no";
foreach my $op (@ops) {
    my $lop = lc($op);
    $ua1->get_ok("http://localhost/host/action?hostop=$op&hostname=$nxname&v46=1.2.3.4", "$op on non-existing host name");

    if ($lop eq 'check') {
	$ua1->content_contains("$lop '$nxname' succeeded", "[023-I-5] $op on non-existing $nxname performed");
	$ua1->content_contains("host does not exist.", "[023-I-5] $op on non-existing $nxname performed - not found");
    }  else {
	$ua1->content_contains("$lop '$nxname' failed", "[023-I-5] $op on non-existing $nxname performed");
	$ua1->content_contains("Host [$nxname] not found in the registry.", "[023-I-5] $op on non-existing host name failed - not found");
    }
}

#print $ua1->content;

# ok name
$ua1->get_ok("http://localhost/host/action/?hostop=Check&hostname=$hn", 'Check created host ...');
$ua1->content_contains("check '$hn' succeeded", "[023-I-5] Host check on $hn OK");


### 
# Info
#

# Look it up, with complete objdetails to also get the Client ID
# Also verify proper sponsorship, rega for $hn, and reg2 for biff.uninett.no
foreach my $h ('biff.uninett.no', $hn) {
    my $expregid;
    if ($h eq $hn) {
	$expregid = $rega;
    } else {
	$expregid = 'reg2';
    }
    $ua1->get_ok("http://localhost/host/action/?hostop=Info&hostname=$h&objdetails=on&sponsoringclientid=$expregid", "Info on host $h with details ...");
    $ua1->content_contains("info '$h' succeeded", "[023-I-2] Host Info up on $h");

    my $c = $ua1->content(format => 'text');
    my $regexp = qr/(Client ID:)(reg\d+)/;
    my $regid;
    if ($c =~ $regexp) {
	$regid = $2;
    }
    ok ($regid && $regid eq $expregid, "[023-B-1] Correct host object sponsorship $expregid for host $h verified");
}


### 
# Update, only ip-address and contact can be changed
#
# Another update, but with no changes
$ua1->get_ok("http://localhost/host/action/?hostop=Update&hostname=$hn&v46=$v4a&v46=$v6a", "Update host $hn without changes ...");
$ua1->content_contains("Nothing to do", "[023-J-5] Update host '$hn' without changes");

# Add a contact
$ua1->get_ok("http://localhost/host/action/?hostop=Update&hostname=$hn&v46=$v4a&v46=$v6a&contact=$srid_p", "Update host $hn with contact ...");
$ua1->content_contains("update '$hn' succeeded", "[023-J-5] Update host '$hn' with contact");

# Change to non-existing contact
$ua1->get_ok("http://localhost/host/action/?hostop=Update&hostname=$hn&v46=$v4a&v46=$v6a&contact=$nxc", "Update host $hn with non-existing contact ...");
$ua1->content_contains("update '$hn' failed", "[023-J-5] Update host '$hn' with non-existing contact");
$ua1->content_contains("Contact with handle [$nxc] not found.", "[023-J-5] Update host '$hn' with non-existing contact - not found");

# Change to illegal contact
$ua1->get_ok("http://localhost/host/action/?hostop=Update&hostname=$hn&v46=$v4a&v46=$v6a&contact=$illc", "Update host $hn with illegal contact ...");
$ua1->content_contains("Illegal Contact ID '$illc'", "[023-J-5] Update host '$hn' with illegal contact");

# Change to a contact owned by another registrar, shall be permitted 
$ua1->get_ok("http://localhost/host/action/?hostop=Update&hostname=$hn&v46=$v4a&v46=$v6a&contact=$srid_p2", "Update host $hn with contact owner by other registrar ...");
#print $ua1->content;
$ua1->content_contains("update '$hn' succeeded","[023-J-5] Update host '$hn' with with contact owner by other registrar");


# Update a name server $hn3 owned by another registrar, should not be found 
$ua1->get_ok("http://localhost/host/action/?hostop=Info&hostname=$hn3&sponsoringclientid=$regb", "Info on host $hn3 owned by other registrar ...");
$ua1->content_contains("info '$hn3' succeeded", "Info on host $hn3 owned by other registrar ...");
#print $ua1->content;
$ua1->get_ok("http://localhost/host/action/?hostop=Update&hostname=$hn3&v46=$v4a&v46=$v6a", "Update host $hn3 owned by other registrar ...");
$ua1->content_contains("update '$hn3' failed","[023-J-5] Update host $hn3 owned by other registrar failed");
$ua1->content_contains("Host [$hn3] not found in the registry.","[023-J-5] Update host $hn3 owned by other registrar failed - not found in registry");
#print $ua1->content;

###
# Pending cases
# Do an Info first
$ua1->get_ok("http://localhost/host/action/?hostop=Info&hostname=$hn", "Info on host $hn ...");
$ua1->content_contains("info '$hn' succeeded", "Info on host $hn ...");

# Modify the host name. This Update will go in pending-queue, so the Info will return old values
my $newhn="new-" . $hn;
$ua1->get_ok("http://localhost/host/action/?hostop=Update&hostname=$newhn&v46=$v4a&v46=$v6a", "Update host with new name $newhn...");
$ua1->content_contains("update '$hn' succeeded", "[023-J-5] Update to '$newhn' in pending");
#print $ua1->content;
$ua1->content_contains("The ticket has been queued for DNS check.", "[023-JJ-3a] Update '$newhn' name in queue for DNS check");

# Modify the ipv6 address. Note: this Update will go in pending-queue, so the Info will return old values
$v6a='2001:700:1:60:215:f2ff:fe3e:fe64';
$ua1->get_ok("http://localhost/host/action/?hostop=Update&hostname=$hn&v46=$v4a&v46=$v6a", "Update host...");
$ua1->content_contains("update '$hn' succeeded", "[023-J-5] Update '$hn' with new ip-address");
$ua1->content_contains("The ticket has been queued for DNS check.", "[023-JJ-3b] Update '$hn' in  in queue for DNS check");


if ($uaf) {
    ######################################################
    #
    # Facets
    #
    ######################################################

    #
    # 1) Facet: impersonate_registrar
    #
    my $FACETS = "facetmenu=on&vfyfacets=on&impersonate_registrar=$rega";

    ####
    # Check
    # 1) check if host $hn exist under $rega, it shall exist already
    # 2) check if host $hn exist under $regf, it shall not exist

    $uaf->get_ok("http://localhost/host/action/?hostop=Check&hostname=$hn");
    $uaf->content_contains("check '$hn' succeeded", "[023-PP-1a] Host check on $hn OK");
    $uaf->content_contains("host does not exist.", "[023-PP-1a] Host check on $hn - not exist");

    $uaf->get_ok("http://localhost/host/action/?hostop=Check&hostname=$hn&$FACETS");
    $uaf->content_contains("check '$hn' succeeded", "[023-PP-1a] Host check on $hn - and under $rega - with facets - OK");
    $uaf->content_contains("host exists.", "[023-PP-1a] Host check on $hn - and under $rega - with facets - does exist");

    # create a normal host $hn under regf, it already exists under rega
    $uaf->get_ok("http://localhost/host/action/?hostop=Create&hostname=$hn&v46=$v4a&v46=$v6a", "Create host '$hn' under $regf");
    $uaf->content_contains("create '$hn' succeeded", "[023-J-2] Created host '$hn' under $regf");


    ####
    # Create host as regf, but impose as rega by facet
    my $hf1="ns1.facet-host-$sfx.no";
    $uaf->get_ok("http://localhost/host/action/?hostop=Create&hostname=$hf1&v46=$v4a&v46=$v6a&$FACETS", "Create host '$hf1', as $regf but impersonate $rega");
    $uaf->content_contains("create '$hf1' succeeded", "[023-PP-1a] Created host '$hf1', as $regf but impersonated $rega");


    ####
    # do an info and verify that the facet host were created with rega as sponsor, use sponsoringclientid=rega
    my ($res, $regid) = do_info($uaf, $hf1, $rega);
    ok ($res && $regid && $regid eq $rega, "[023-B-1] Correct host object sponsorship $rega for host '$hf1' verified");

    ####
    # Info on $hn without facets - shall return regf as sponsor
    #
    ($res, $regid) = do_info($uaf, $hn, undef);
    ok ($res && $regid && $regid eq $regf, "[023-B-1] Correct host object sponsorship $regf for host '$hn' - no facets - verified");

    ####
    # Info on $hn with facets - shall return rega as sponsor
    ($res, $regid) = do_info($uaf, $hn, undef, $FACETS);
    ok ($res && $regid && $regid eq $rega, "[023-PP-1a] Correct host object sponsorship $rega for host '$hn' - with facets - verified");


    ####
    # Update - remove a ipv6-address  - of $hn2 which is sponsored by rega

    ####
    # First an info on $hf1 to get the current data
    ($res, $regid) = do_info($uaf, $hf1, $rega);
    ok ($res && $regid && $regid eq $rega, "[023-PP-1a] Prepare update, correct host object sponsorship $rega for host '$hf1' - no facets - verified");

    # 1) without facet, shall be rejected
    $uaf->get_ok("http://localhost/host/action/?hostop=Update&hostname=$hf1&v46=$v4a", "Update host $hf1 and remove the $v6a address");
    $uaf->content_contains("update '$hf1' failed", "[023-J-5] Update host '$hf1' and remove the $v6a address - rejected");

    # 2) with facet, shall be accepted
    $uaf->get_ok("http://localhost/host/action/?hostop=Update&hostname=$hf1&v46=$v4a&$FACETS", "Update host $hf1 and remove the $v6a address - with facet");
    $uaf->content_contains("update '$hf1' succeeded", "[023-PP-1a] Update host '$hf1' and remove the $v6a address - with facet - accepted");

    ####
    # Delete $hf1, sponsored by rega

    # 1) without facet, shall be rejected
    $uaf->get_ok("http://localhost/host/action/?hostop=Delete&hostname=$hf1", "Delete others host $hf1 without facet, shall be rejected ...");
    $uaf->content_contains("delete '$hf1' failed", "[023-PP-1a] Delete others host '$hf1' without facet rejected - OK");

    # 2) with facet, shall be accepted
    $uaf->get_ok("http://localhost/host/action/?hostop=Delete&hostname=$hf1&$FACETS", "Delete others host $hf1 with facet, shall be accepted ...");
    $uaf->content_contains("delete '$hf1' succeeded", "[023-PP-1a] Delete others host '$hf1' by facet OK");
}

#################################
#
# Cleanup
# 
#################################

# delete the hosts
foreach my $h ($hn, $hn2) {
    # delete, is NOT sponsor
    $ua2->get_ok("http://localhost/host/action/?hostop=Delete&hostname=$h", "Delete host $h, but not sponsor");
    $ua2->content_contains("delete '$h' failed", "[023-J-12] Delete '$h' was rejected, not sponsor");

    # delete, is sponsor
    $ua1->get_ok("http://localhost/host/action/?hostop=Delete&hostname=$h", "Delete host $h...");
    $ua1->content_contains("delete '$h' succeeded", "[023-J-12] Delete '$h' OK");

}
# delete, is sponsor
$ua1->get_ok("http://localhost/host/action/?hostop=Delete&hostname=$hnet", "Delete host $hnet");
$ua1->content_contains("delete '$hnet' succeeded", "[023-J-12] Delete '$hnet' OK");

$ua2->get_ok("http://localhost/host/action/?hostop=Delete&hostname=$hn3", "Delete host $hn3 ...");
$ua2->content_contains("delete '$hn3' succeeded", "[023-J-12] Delete '$hn3' OK");

if ($uaf) {
    $uaf->get_ok("http://localhost/host/action/?hostop=Delete&hostname=$hn", "Delete host $hn ...");
    $uaf->content_contains("delete '$hn' succeeded", "[023-PP-1a] Delete '$hn' OK");
}

####
# delete the contacts
$ua1->get_ok("http://localhost/contact/action?contactop=Delete&srid=$srid_p", "Delete '$srid_p'...");
$ua1->content_contains("delete '$srid_p' succeeded", "Delete '$srid_p' contact");

$ua2->get_ok("http://localhost/contact/action?contactop=Delete&srid=$srid_p2", "Delete '$srid_p2'...");
$ua2->content_contains("delete '$srid_p2' succeeded", "Delete '$srid_p2' contact");



###
# do a host info on a host, if sponsoringclientid is specified, send it,
# if a facet is specified, send it
# return ok or not, and clID value if found

sub do_info {
    my ($ua, $hn, $sponsoringclientid, $facet) = @_;

    my$SPONSORINGCLIENTID = "";
    $SPONSORINGCLIENTID   = "&sponsoringclientid=" . $sponsoringclientid if ($sponsoringclientid);

    my $FACETS = "";
    my $factxt = "";
    if ($facet) {
	$FACETS= "&" . $facet;
	$factxt = "facet";
    }

    $ua->get_ok("http://localhost/host/action/?hostop=Info&hostname=$hn&objdetails=on$SPONSORINGCLIENTID$FACETS", "Info on $factxt host $hn with details ...");

    my ($result, $regid);

    if ($result = $ua->content_contains("info '$hn' succeeded", "[023-PP-1] Info on $factxt host '$hn' OK")) {
	my $c = $ua->content(format => 'text');
	my $regexp = qr/(Client ID:)(reg\d+)/;

	if ($c =~ $regexp) {
	    $regid = $2;
	}
    }

    return $result, $regid;
}
