#!/usr/bin/perl
use strict;
use warnings;
use Test::More "no_plan";
use Test::WWW::Mechanize::Catalyst 'EppClient';
use URI::Escape;
use HTML::Entities;
use Encode;
use EppClient::Controller::Util;
use EppClient::InitTests;
use YAML;

my $tc = EppClient::InitTests::get_test_config;
my $t = $tc->{t};

my $reg0 = $t->{reg0};
my $pw0  = $t->{pw0};
my $rega = $t->{rega};
my $regb = $t->{regb};
my $pwa  = $t->{pwa};
my $pwb  = $t->{pwb};

my $cf = EppClient->config();

my $today    = EppClient::Controller::Util::timestamp_as_date(time);
my $tomorrow = EppClient::Controller::Util::timestamp_as_date(time + (60*60*24));
my $future   = EppClient::Controller::Util::timestamp_as_date(time + (60*60*48));
my $past     = EppClient::Controller::Util::timestamp_as_date(time - (60*60*48));

# string to make names unique, todays date + process id
my $sfx = "$today-$$";

my $ua1 = Test::WWW::Mechanize::Catalyst->new;
my $ua2 = Test::WWW::Mechanize::Catalyst->new;
my $ua0;
if ($reg0) {
    $ua0 = Test::WWW::Mechanize::Catalyst->new;
    is(1, 1, "Running with reg0 tests");
} else {
    is(1, 1, "Running without reg0 tests");
}

$ua1->get_ok("http://localhost/", 'get localhost OK');

is($ua1->ct, "text/html", 'text/html OK');

$ua1->title_is("Norid EPP Client", "On the EPP Client's root page");

$ua1->content_contains("You need to log in to use this application", "Yup, on the login page");

### login with correct pw
# login with correct pw
$ua1->get_ok("http://localhost/login?command=Login&clid=$rega&pw=$pwa", 'Login OK');
$ua2->get_ok("http://localhost/login?command=Login&clid=$regb&pw=$pwb", 'Login OK');
if ($ua0) {
    $ua0->get_ok("http://localhost/login?command=Login&clid=$reg0&pw=$pw0", 'Login OK');
}

$ua1->content_contains("Main menu", "Main menu found");
$ua2->content_contains("Main menu", "Main menu found");
if ($ua0) {
    $ua0->content_contains("Main menu", "Main menu found");
}

### Create 3 hosts
my $hostname_1="ns1.domcre-barks$sfx.no";
my $v4_1='1.1.1.1';
$ua1->get_ok("http://localhost/host/action/?hostop=Create&hostname=$hostname_1&v46=$v4_1", "Create host 1: $hostname_1 under $rega" );
$ua1->content_contains("create '$hostname_1' succeeded", "Create host 1: $hostname_1 succeeded under $rega");

$ua2->get_ok("http://localhost/host/action/?hostop=Create&hostname=$hostname_1&v46=$v4_1", "Create host 1: $hostname_1 under $regb" );
$ua2->content_contains("create '$hostname_1' succeeded", "Create host 1: $hostname_1 succeeded under $regb");

my $hostname_2="ns2.domcre-barks$sfx.no";
my $v4_2='2.2.2.2';
$ua1->get_ok("http://localhost/host/action/?hostop=Create&hostname=$hostname_2&v46=$v4_2", "Create host 2: $hostname_2 under $rega");
$ua1->content_contains("create '$hostname_2' succeeded", "Create host 2: $hostname_2 succeeded under $rega");

$ua2->get_ok("http://localhost/host/action/?hostop=Create&hostname=$hostname_2&v46=$v4_2", "Create host 2: $hostname_2 under $regb");
$ua2->content_contains("create '$hostname_2' succeeded", "Create host 2: $hostname_2 succeeded under $regb");


my $hostname_3="ns3.domcre-barks$sfx.no";
$v4_2='3.3.3.3';
$ua1->get_ok("http://localhost/host/action/?hostop=Create&hostname=$hostname_3&v46=$v4_2", "Create host 3: $hostname_3 under $rega");
$ua1->content_contains("create '$hostname_3' succeeded", "Create host 3: $hostname_3 succeeded under $rega");

$ua2->get_ok("http://localhost/host/action/?hostop=Create&hostname=$hostname_3&v46=$v4_2", "Create host 3: $hostname_3 under $regb");
$ua2->content_contains("create '$hostname_3' succeeded", "Create host 3: $hostname_3 succeeded under $regb");


### Create some person contacts
my $contactname="Carl Domcre Barks-$sfx";
my $type='person';
my $cc='NO';
my $pc='NO-7004';
my $city='Trondheim';
my $email='carl.barks@data.no';
my $PID1 = $t->{pidno1};
my $identitytype_p = 'anonymousPersonIdentifier';
my $xtr="";

unless ($cf->{client}->{old_data_model}) {
    $contactname = $t->{pidname1};
    $xtr="&identitytype=$identitytype_p&identityvalue=$PID1";
}

# Create a couple of tech role
my $rtype='role';
my $rcontactname="Role Carl Domcre Barks-$sfx";

$ua1->get_ok("http://localhost/contact/action?contactop=Create&contactname=$rcontactname&type=$rtype&cc=$cc&pc=$pc&city=$city&email=$email", "Create Role under $rega");
my $srid_r = $ua1->form_number('1')->value('srid');
$ua1->content_contains("create '$rcontactname' succeeded", "[023-J-3] Created '$rcontactname' ($srid_r) of type $rtype under $rega OK");

$ua1->get_ok("http://localhost/contact/action?contactop=Create&contactname=$rcontactname&type=$rtype&cc=$cc&pc=$pc&city=$city&email=$email", "Create Role 2 under $rega");
my $srid_rr = $ua1->form_number('1')->value('srid');
$ua1->content_contains("create '$rcontactname' succeeded", "[023-J-3] Created '$rcontactname' 2 ($srid_rr) of type $rtype under $rega OK");


$ua2->get_ok("http://localhost/contact/action?contactop=Create&contactname=$rcontactname&type=$rtype&cc=$cc&pc=$pc&city=$city&email=$email", "Create Role under $regb");
my $srid_rb = $ua2->form_number('1')->value('srid');
$ua2->content_contains("create '$rcontactname' succeeded", "[023-J-3] Created '$rcontactname' ($srid_rb) of type $rtype under $regb OK");


$ua1->get_ok("http://localhost/contact/action?contactop=Create&contactname=$contactname&type=$type&cc=$cc&pc=$pc&city=$city&email=$email$xtr", "Create Person under $rega");

my $srid_p = $ua1->form_number('1')->value('srid');
$ua1->content_contains("create '$contactname' succeeded", "[023-J-3] Created '$contactname' ($srid_p) of type $type under $rega OK");

$ua1->get_ok("http://localhost/contact/action?contactop=Create&contactname=$contactname&type=$type&cc=$cc&pc=$pc&city=$city&email=$email$xtr", "Create Person under $rega");
my $srid_pp = $ua1->form_number('1')->value('srid');
$ua1->content_contains("create '$contactname' succeeded", "[023-J-3] Created '$contactname' ($srid_pp) of type $type under $rega OK");


$ua1->get_ok("http://localhost/contact/action?contactop=Create&contactname=$contactname&type=$type&cc=$cc&pc=$pc&city=$city&email=$email$xtr", "Create Person under $rega");
my $srid_ppp = $ua1->form_number('1')->value('srid');
$ua1->content_contains("create '$contactname' succeeded", "[023-J-3] Created '$contactname' ($srid_ppp) of type $type under $rega OK");

$ua2->get_ok("http://localhost/contact/action?contactop=Create&contactname=$contactname&type=$type&cc=$cc&pc=$pc&city=$city&email=$email$xtr", "Create Person under $regb");
my $srid_p2 = $ua2->form_number('1')->value('srid');
$ua2->content_contains("create '$contactname' succeeded", "[023-J-3] Created '$contactname' ($srid_p2) of type $type under $regb OK");

###
# Create: A private person contact
my $PID      = $t->{pidno2};
my $pid_name = $t->{pidname2};

$ua1->get_ok("http://localhost/contact/action?contactop=Create&contactname=$pid_name&type=$type&identitytype=$identitytype_p&identityvalue=$PID&cc=$cc&pc=$pc&city=$city&email=$email", "Create anonymeous private person under $rega");
my $srid_priv = $ua1->form_number('1')->value('srid');
$ua1->content_contains("create '$pid_name' succeeded", "[023-J-?] Created private '$pid_name' ($srid_priv) of type $identitytype_p $rega OK");


### Create a Brreg organization contact
#my $contactname_1="Carl Domcre Barks & Sons-$sfx";  
my $contactname_1 = $t->{orgnamea1};
my $contactname_2 = $t->{orgnamea2};
my $econtactname_1=encode_entities($contactname_1);
my $econtactname_2=encode_entities($contactname_2);
             
my $type_1='organization';
my $identitytype_1='organizationNumber';

# Need a unique and not yet used orgno for a Brreg-org create
my $orgno = $t->{orgnoa1};
my $identityvalue_1 = sprintf("%09d", $orgno);
my $cc_1='NO';
my $pc_1='NO-7004';
my $city_1='Trondheim';
my $email_1='barks@data.no';
my $lcname_1 = $t->{legalcname1};

my $srid_o;

# Create regexp to match: Contact with identity [organizationNumber:123456888] already exists as [CBS33O]
my $regexp = qr/(Contact with identity \[$identitytype_1:$identityvalue_1\] already exists as \[(\w+)\])/;

if ($cf->{client}->{new_data_model}) {
    # Use org name and legalc name in an org
    $ua1->get_ok("http://localhost/contact/action?contactop=Create&contactorg=".uri_escape($contactname_1)."&contactname=$lcname_1&type=$type_1&cc=$cc_1&pc=$pc_1&city=$city_1&email=$email_1&identitytype=$identitytype_1&identityvalue=$identityvalue_1", "Create Organization under $rega");
    
    $srid_o = $ua1->form_number('1')->value('srid');
    
    $ua1->content_contains("create '$lcname_1' succeeded", "[023-J-3] Created '$contactname_1' ($srid_o) of type $type_1 under $rega");

} else {

    $ua1->get_ok("http://localhost/contact/action?contactop=Create&contactname=".uri_escape($contactname_1)."&type=$type_1&cc=$cc_1&pc=$pc_1&city=$city_1&email=$email_1&identitytype=$identitytype_1&identityvalue=$identityvalue_1", "Create Organization under $rega");
    
    if ($ua1->content =~ $regexp) {
	#print "ok $1\n";
	$srid_o = $2;
	ok($1, "Contact with identity [$identitytype_1:$identityvalue_1] already exists as [$srid_o] under $rega");
    } else {
	$srid_o = $ua1->form_number('1')->value('srid');
	$ua1->content_contains("create '$econtactname_1' succeeded", "[023-J-3] Created '$contactname_1' ($srid_o) of type $type_1 under $rega");
	    
    }
}

my $srid_o2;

if ($cf->{client}->{new_data_model}) {
    # Use org name and legalc name in an org
    $ua2->get_ok("http://localhost/contact/action?contactop=Create&contactorg=".uri_escape($contactname_1)."&contactname=$lcname_1&type=$type_1&cc=$cc_1&pc=$pc_1&city=$city_1&email=$email_1&identitytype=$identitytype_1&identityvalue=$identityvalue_1", "Create Organization under $regb");

    $srid_o2 = $ua2->form_number('1')->value('srid');
    $ua2->content_contains("create '$lcname_1' succeeded", "[023-J-3] Created '$contactname_1' ($srid_o2) of type $type_1 under $regb");
    
} else {
    $ua2->get_ok("http://localhost/contact/action?contactop=Create&contactname=".uri_escape($contactname_1)."&type=$type_1&cc=$cc_1&pc=$pc_1&city=$city_1&email=$email_1&identitytype=$identitytype_1&identityvalue=$identityvalue_1", "Create Organization under $regb");
    $srid_o2 = $ua2->form_number('1')->value('srid');
    # Create regexp to match: Contact with identity [organizationNumber:123456888] already exists as [CBS33O]
    $regexp = qr/(Contact with identity \[$identitytype_1:$identityvalue_1\] already exists as \[(\w+)\])/;
    if ($ua2->content =~ $regexp) {
	#print "ok $1\n";
	$srid_o2 = $2;
	ok($1, "Contact with identity [$identitytype_1:$identityvalue_1] already exists as [$srid_o2] under $regb");
    } else {   
	$ua2->content_contains("create '$econtactname_1' succeeded", "[023-J-3] Created '$contactname_1' ($srid_o2) of type $type_1 under $regb");
    }
}

# another org under $rega
my $orgno3 = $t->{orgnoa2};
my $identityvalue_3 = sprintf("%09d", $orgno3);
my $lcname_2 = $t->{legalcname2};

#print $ua1->content;

my $srid_o3;

if ($cf->{client}->{new_data_model}) {
    # Use org name and legalc name in an org
    $ua1->get_ok("http://localhost/contact/action?contactop=Create&contactorg=".uri_escape($contactname_2)."&contactname=$lcname_2&type=$type_1&cc=$cc_1&pc=$pc_1&city=$city_1&email=$email_1&identitytype=$identitytype_1&identityvalue=$identityvalue_3", "Create Organization under $rega");

    $srid_o3 = $ua1->form_number('1')->value('srid');
    $ua1->content_contains("create '$lcname_2' succeeded", "[023-J-3] Created '$contactname_2' ($srid_o3) of type $type_1 under $rega");
    
} else {
    $ua1->get_ok("http://localhost/contact/action?contactop=Create&contactname=".uri_escape($contactname_2)."&type=$type_1&cc=$cc_1&pc=$pc_1&city=$city_1&email=$email_1&identitytype=$identitytype_1&identityvalue=$identityvalue_3", "Create Organization under $rega");
    
    $srid_o3 = $ua1->form_number('1')->value('srid');
    # Create regexp to match: Contact with identity [organizationNumber:123456888] already exists as [CBS33O]
    $regexp = qr/(Contact with identity \[$identitytype_1:$identityvalue_3\] already exists as \[(\w+)\])/;
    if ($ua1->content =~ $regexp) {
	#print "ok $1\n";
	$srid_o3 = $2;
	ok($1, "Contact with identity [$identitytype_1:$identityvalue_3] already exists as [$srid_o3] under $rega");
    } else {   
	$ua1->content_contains("create '$econtactname_2' succeeded", "[023-J-3] Created '$contactname_2' ($srid_o3) of type $type_1 under $rega");
    }
}

if ($cf->{client}->{old_data_model}) {
    ### Add the Organization contact to the Person contact
    $ua1->get_ok("http://localhost/contact/action?contactop=Info&srid=$srid_p", "Info on $srid_p");
    $ua1->content_contains("info '$srid_p' succeeded", "Info on $srid_p ok");
    $ua1->get_ok("http://localhost/contact/action?contactop=Update&contactname=$contactname&type=$type&cc=$cc&pc=$pc&city=$city&email=$email&srid=$srid_p&organization=$srid_o", "Add the Organization contact: $srid_o to the Person contact: $srid_p");
    $ua1->content_contains("update '$srid_p' succeeded", "[023-J-6] Update contact $srid_p with Related Org $srid_o OK");

    $ua1->get_ok("http://localhost/contact/action?contactop=Info&srid=$srid_pp", "Info on $srid_pp");
    $ua1->content_contains("info '$srid_pp' succeeded", "Info on $srid_pp ok");
    $ua1->get_ok("http://localhost/contact/action?contactop=Update&contactname=$contactname&type=$type&cc=$cc&pc=$pc&city=$city&email=$email&srid=$srid_pp&organization=$srid_o", "Add the Organization contact: $srid_o to the Person contact: $srid_pp");
    $ua1->content_contains("update '$srid_pp' succeeded", "[023-J-6] Update contact $srid_pp with Related Org $srid_o OK");


    $ua1->get_ok("http://localhost/contact/action?contactop=Info&srid=$srid_ppp", "Info on $srid_ppp");
    $ua1->content_contains("info '$srid_ppp' succeeded", "Info on $srid_ppp ok");

    $ua1->get_ok("http://localhost/contact/action?contactop=Update&contactname=$contactname&type=$type&cc=$cc&pc=$pc&city=$city&email=$email&srid=$srid_ppp&organization=$srid_o3", "Add the Organization contact: $srid_o3 to the Person contact: $srid_ppp");
    $ua1->content_contains("update '$srid_ppp' succeeded", "[023-J-6] Update contact $srid_ppp with Related Org $srid_o3 OK");
    
    $ua2->get_ok("http://localhost/contact/action?contactop=Info&srid=$srid_p2", "Info on $srid_p2");
    $ua2->content_contains("info '$srid_p2' succeeded", "Info on $srid_p2 ok");
    $ua2->get_ok("http://localhost/contact/action?contactop=Update&contactname=$contactname&type=$type&cc=$cc&pc=$pc&city=$city&email=$email&srid=$srid_p2&organization=$srid_o2", "Add the Organization contact: $srid_o2 to the Person contact: $srid_p2");
    $ua2->content_contains("update '$srid_p2' succeeded", "[023-J-6] Update contact $srid_p2 with Related Org $srid_o2 OK");

}

#####
# ILLEGAL PARS:
# The various domain parameters
#

# Applicant dataset arguments.
my $versionnumber = "2.0";
my $acceptdate = $past . "T14:30:48Z";
my $acceptname = "Tante Sofie";
my $appldataset_args = "versionnumber=$versionnumber&acceptdate=$acceptdate&acceptname=$acceptname&vfyapplicantdatasetupdate=on&objdetails=on";


### For create domains, use date and pid to make it pretty uniq
my $domainname="domcre-barks-$sfx-$rega.no";
my $domainname_priv="domcre-barks-$sfx-$rega.priv.no";
my $domainname_priv2="domcre-barks-$sfx-$rega-2.priv.no";

my $domainname2="domcre-barks$sfx-$rega-withdrawtest.no";
my $domainnameb="domcre-barks$sfx-$regb.no";
my $nsset_1="ns1.domcre-barks$sfx.no";
my $nsset_2="ns2.domcre-barks$sfx.no";
my $nsset_3="ns3.domcre-barks$sfx.no";
my $registrant=$srid_o;	# Barks Org
my $admin=$srid_p;   	# Person 
my $tech=$srid_r;	# Role
my $pw='1234qwer';      # Auth Info (pw)

my $registrantb=$srid_o2;   ## Barks Org
my $adminb=$srid_p2;        # Person 
my $techb=$srid_rb;	    # Role


### 
## Normal domains
my $adminstr  = "";
my $adminbstr = "";

if ($cf->{client}->{old_data_model}) {
    $adminstr  = "&admin=$admin";
    $adminbstr = "&admin=$adminb";
}

# domain name missing
$ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=&nsset=$nsset_1&nsset=$nsset_2&registrant=$registrant$adminstr&tech=$tech&pw=$pw", "Create domain without name shall fail");
$ua1->content_contains("create: mandatory parameter 'name' not specified", "[023-J-1] Create without domain name failed");

# domain name illegal
my $domill='ole_dole.no';
$ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=$domill&nsset=$nsset_1&nsset=$nsset_2&registrant=$registrant$adminstr&tech=$tech&pw=$pw", "Create domain with illegal name shall fail");
$ua1->content_contains("create: illegal domainname: '$domill'", "[023-J-1] Create with illegal domain name $domill failed");

# name servers are missing
$ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=$domainname&nsset=&registrant=$registrant$adminstr&tech=$tech&pw=$pw", "Create domain with missing host name shall fail");
$ua1->content_contains("create: mandatory parameter 'name server' not specified", "[023-J-1] Create with missing host name failed");

# name server name illegal
$nsset_1="ns1.b_arks$sfx.no";
$ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=$domainname&nsset=$nsset_1&nsset=$nsset_2&registrant=$registrant$adminstr&tech=$tech&pw=$pw", "Create domain with illegal host name shall fail");
$ua1->content_contains("create: illegal hostname: '$nsset_1'", "[023-J-1] Create with illegal host name failed");
$nsset_1="ns1.domcre-barks$sfx.no";

# too many name servers, try with 10 which should be above the practical limit (8)
my $nsset_l;
map {$nsset_l .= "&nsset=ns$_.many$sfx.no" } (1..10);
#print "nsset_l: $nsset_l\n";
$ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=$domainname$nsset_l&registrant=$registrant$adminstr&tech=$tech&pw=$pw&$appldataset_args", "Create domain with too many name servers shall fail");

#print $ua1->content;
$ua1->content_contains("Too many nameservers, maximum is [8].", "[023-J-1] Create with too many name servers failed");

# registrant is missing
$ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=$domainname&nsset=$nsset_1&nsset=$nsset_2&registrant=$adminstr&tech=$tech&pw=$pw&$appldataset_args", "Create domain without registrant shall fail");
$ua1->content_contains("create: mandatory parameter 'registrant' not specified", "[023-J-1] Create domain without registrant failed");

unless ($cf->{client}->{new_data_model}) {
 # admin is missing (old data model only)
 $ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=$domainname&nsset=$nsset_1&nsset=$nsset_2&registrant=$registrant&admin=&tech=$tech&pw=$pw&$appldataset_args", "Create domain without admin shall fail");
 $ua1->content_contains("create: mandatory parameter 'admin' contact not specified", "[023-J-1] Create domain without admin failed");
}
    
# tech is missing
$ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=$domainname&nsset=$nsset_1&nsset=$nsset_2&registrant=$registrant$adminstr&tech=&pw=$pw&$appldataset_args", "Create domain without tech shall fail");
$ua1->content_contains("create: mandatory parameter 'tech' contact not specified", "[023-J-1] Create domain without tech failed");

# registrant is illegal
$registrant='ILLE12';
$ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=$domainname&nsset=$nsset_1&nsset=$nsset_2&registrant=$registrant$adminstr&tech=$tech&pw=$pw&$appldataset_args", "Create domain with illegal registrant shall fail");

$ua1->content_contains("create: Invalid Registrant ID, must be an organization contact: '$registrant'", "[023-J-1] Create domain with illegal registrant failed");
$registrant=$srid_o;

if ($cf->{client}->{old_data_model}) {
    # admin is illegal, only in old model
    $admin='ILLE12';
    $adminstr = "&admin=$admin";

    $ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=$domainname&nsset=$nsset_1&nsset=$nsset_2&registrant=$registrant$adminstr&tech=$tech&pw=$pw&$appldataset_args", "Create domain with illegal admin shall fail");
    $ua1->content_contains("create: Invalid Admin contact ID: '$admin'", "[023-J-1] Create domain with illegal admin failed");
    
    $admin=$srid_p;  
    $adminstr = "&admin=$admin";
}

# tech is illegal
$tech='ILLE12';
$ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=$domainname&nsset=$nsset_1&nsset=$nsset_2&registrant=$registrant$adminstr&tech=$tech&pw=$pw&$appldataset_args", "Create domain with illegal tech shall fail");
$ua1->content_contains("create: Invalid Tech contact ID: '$tech'", "[023-J-1] Create domain with illegal tech failed");

$tech=$srid_r;  

# Illegal authInfo
# Too short
my $illpw = "abc";
$ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=$domainname&nsset=$nsset_1&nsset=$nsset_2&registrant=$registrant$adminstr&tech=$tech&pw=$illpw&$appldataset_args", "Create domain with illegal authInfo shall fail (too short)");
$ua1->content_contains("Bad authinfo value. The password must contain 8 to 16 characters.", "[023-J-1] Create domain with illegal AuthInfo failed (too short)");

#print $ua1->content;

# Too long, 22 chars
$illpw = "1234567890ABCDEFGHIJK2";

#print "illpw: @illpw\n";
$ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=$domainname&nsset=$nsset_1&nsset=$nsset_2&registrant=$registrant$adminstr&tech=$tech&pw=$illpw&$appldataset_args", "Create domain with illegal authInfo shall fail (too long)");
$ua1->content_contains("Bad authinfo value. The password must contain 8 to 16 characters.", "[023-J-1] Create domain with illegal AuthInfo failed (too long)");

#

# tech contact is under other sponsor
$ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=$domainname&nsset=$nsset_1&nsset=$nsset_2&registrant=$registrant$adminstr&tech=$techb&pw=$pw&$appldataset_args", "Create domain with tech under other sponsor shall fail");
$ua1->content_contains("create '$domainname' failed.", "[023-J-1] Create domain with tech contact under other sponsor failed");


# Create domain with renew/duration period set to other than 12 months
my $duration=10;
$ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=$domainname&nsset=$nsset_1&nsset=$nsset_2&registrant=$registrant$adminstr&tech=$tech&pw=$pw&duration=$duration&$appldataset_args", "Create domain $domainname with implicit renew and period set to other than 12 months - shall be rejected");
$ua1->content_contains("create '$domainname' failed", "[023-C-5] Create domain $domainname with implicit renew and period set to other than 12 months - was rejected");
$ua1->content_contains("Invalid duration", "[023-C-5] Create domain $domainname with implicit renew and period set to other than 12 months - was rejected");

if ($cf->{client}->{allow_private_no_registrant}) {

    my $p_domainname = 'privtest-' . $domainname;

    ##############
    # Create private domain under .no - shall be accepted
    $ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=$p_domainname&nsset=$nsset_1&nsset=$nsset_2&registrant=$srid_priv&tech=$tech&pw=$pw&$appldataset_args", "Create .no domain with person registrant, shall succeed");
    #print $ua1->content;
    $ua1->content_contains("create '$p_domainname' succeeded", "[023-J-?] Create .no domain with person registrant");
} else {
    ##############
    # Create private domain under .no - shall be rejected
    $ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=$domainname&nsset=$nsset_1&nsset=$nsset_2&registrant=$srid_priv&&tech=$tech&pw=$pw&$appldataset_args", "Create .no domain with person registrant, shall fail");

    $ua1->content_contains("create: Private person as a registrant is not permitted: '$srid_priv'", "[023-J-?] Create .no domain with person registrant");
} 

# Create private domain under priv.no, try to add an admin contact, shall be rejected
$ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=$domainname_priv2&nsset=$nsset_1&nsset=$nsset_2&registrant=$srid_priv&admin=$admin&tech=$tech&pw=$pw&$appldataset_args", "Create priv.no domain with person registrant and admin contact, shall be rejected");
$ua1->content_contains("create: Invalid Admin contact ID: '$admin', admin is not allowed", "[023-J-?] Try to create priv.no domain with admin contact - $domainname_priv2 rejected");

#print $ua1->content;
#sleep 40;

#####
# LEGAL PARS:
# 

##############
# Create private domain under priv.no, no admin contact, shall be OK
$ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=$domainname_priv&nsset=$nsset_1&nsset=$nsset_2&registrant=$srid_priv&&tech=$tech&pw=$pw&$appldataset_args", "Create priv.no domain with person registrant, shall be accepted");
$ua1->content_contains("create '$domainname_priv' succeeded", "[023-J-?] Create priv.no domain $domainname_priv OK");


# Create an IDN domain with 
my $dn_idn = "-$domainname";
my $dn_utf = encode("UTF-8", decode("ISO8859-1", $dn_idn));

$ua1->get_ok(encode("UTF-8", decode("ISO8859-1","http://localhost/domain/action?domainop=Create&domainname=$dn_idn&nsset=$nsset_1&nsset=$nsset_2&registrant=$registrant$adminstr&tech=$tech&pw=$pw&$appldataset_args")), "Create IDN Domain...");
$ua1->content_contains("create '$dn_idn' succeeded", "[023-J-1] Create IDN domain $dn_utf OK");


###
# Create domain under vgs.no, shall be accepted
my $domainname_vgs="domcre-barks$sfx-$rega.vgs.no";
$ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=$domainname_vgs&nsset=$nsset_1&nsset=$nsset_2&registrant=$registrant$adminstr&tech=$tech&pw=$pw&$appldataset_args", "Create domain $domainname_vgs (under vgs.no)");
$ua1->content_contains("create '$domainname_vgs' succeeded", "[023-JJ-2a3] Create $domainname_vgs (under vgs.no) OK");
#$ua1->content_contains("The command has been queued for manual inspection.", "[023-JJ-2a3] Create $domainname_pending was queued for manual inspection");


# Create $regb domain
$ua2->get_ok("http://localhost/domain/action?domainop=Create&domainname=$domainnameb&nsset=$nsset_1&nsset=$nsset_2&registrant=$registrantb$adminbstr&tech=$techb&pw=$pw&$appldataset_args", "Create Domain $domainnameb for $regb ");
#print $ua2->content;
#print $ua2->content;
$ua2->content_contains("create '$domainnameb' succeeded", "[023-J-1] Create $domainnameb for $regb");

##
# Create domain

$ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=$domainname&nsset=$nsset_1&nsset=$nsset_2&registrant=$registrant$adminstr&tech=$tech&pw=$pw&$appldataset_args", "Create domain $domainname");

#print $ua1->content;
#print "Create with: http://localhost/domain/action?domainop=Create&domainname=$domainname&nsset=$nsset_1&nsset=$nsset_2&registrant=$registrant$adminstr&tech=$tech&pw=$pw&$appldataset_args\n";
$ua1->content_contains("create '$domainname' succeeded", "[023-J-1] Create $domainname OK");

# Create same domain once more, shall fail
$ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=$domainname&nsset=$nsset_1&nsset=$nsset_2&registrant=$registrant$adminstr&tech=$tech&pw=$pw&$appldataset_args", "Create same domain $domainname once more, shall fail ");
$ua1->content_contains("create '$domainname' failed", "[023-J-1] Create $domainname again failed - already exists");
$ua1->content_contains("Domain with name [$domainname] already exists in the database", "[023-J-1] Create $domainname again failed - already exists in the registry");


####
# Update domain

# Update with no changes
$ua1->get_ok("http://localhost/domain/action?domainop=Update&domainname=$domainname&nsset=$nsset_1&nsset=$nsset_2&registrant=$registrant$adminstr&tech=$tech&pw=$pw", "Update domain $domainname - no changes shall be rejected");
#print $ua1->content;
$ua1->content_contains("Nothing to do", "[023-J-1] Update $domainname - no changes - was rejected");

# Update most attributes: host and AuthInfo, admin and tech
$pw = 'Newpw123';
my $madminstr="";
if ($cf->{client}->{old_data_model}) {
    $madminstr="&admin=$srid_pp";
}
$ua1->get_ok("http://localhost/domain/action?domainop=Update&domainname=$domainname&nsset=$nsset_1&nsset=$nsset_2&nsset=$nsset_3&registrant=$registrant$madminstr&tech=$srid_rr&pw=$pw", "Update domain $domainname AuthInfo, add a third host, change admin and tech contacts from $srid_p to $srid_pp");
$ua1->content_contains("update '$domainname' succeeded", "[023-J-1] Update domain $domainname AuthInfo, add a third host, change admin and tech contacts from $srid_p to $srid_pp");

# Update and set tech contact to a contact under other sponsor, shall be rejected
$ua1->get_ok("http://localhost/domain/action?domainop=Update&domainname=$domainname&nsset=$nsset_1&nsset=$nsset_2&nsset=$nsset_3&registrant=$registrant$madminstr&tech=$srid_rb&pw=$pw", "Update domain $domainname - set tech contact to a contact under other sponsor - shall be rejected");
$ua1->content_contains("update '$domainname' failed", "[023-J-4] Update domain $domainname - set tech contact to a contact under other sponsor - was rejected");

# Update a domain under other sponsor, shall be rejected
$ua1->get_ok("http://localhost/domain/action?domainop=Info&domainname=$domainnameb&objdetails=on", "Info on $domainnameb");
$ua1->content_contains("info '$domainnameb' succeeded", "[023-I-1] Info on $domainnameb OK");

$ua1->get_ok("http://localhost/domain/action?domainop=Update&domainname=$domainnameb&nsset=$nsset_1&nsset=$nsset_2&nsset=$nsset_3&registrant=$registrantb$madminstr&tech=$srid_rb&pw=$pw&", "Update a domain $domainnameb under other sponsor, shall be rejected");
$ua1->content_contains("update '$domainnameb' failed", "[023-J-4] Update a domain $domainnameb under other sponsor - was rejected");

# Update and try to remove some mandatory attributes (nsset and tech), shall be rejected
$ua1->get_ok("http://localhost/domain/action?domainop=Update&domainname=$domainname&nsset=&registrant=$registrant&admin=&tech=$srid_pp&pw=$pw", "Update domain $domainname - try to remove some mandatory attributes - shall be rejected");
$ua1->content_contains("update: mandatory parameter 'name server' not specified", "[023-J-1] Update domain $domainname - try to remove some mandatory attributes - was rejected");

unless ($cf->{client}->{new_data_model}) {
    $ua1->content_contains("update: mandatory parameter 'admin' contact not specified", "[023-J-1] Update domain $domainname - try to remove some mandatory attributes - was rejected");
}

if ($cf->{client}->{old_data_model} && !$cf->{client}->{new_data_model}) {
    # Update and change admin (legal) contact from srid_pp to srid_ppp (which is related to another organization)
    $ua1->get_ok("http://localhost/domain/action?domainop=Info&domainname=$domainname&objdetails=on", "Info on $domainname");
    $ua1->content_contains("info '$domainname' succeeded", "[023-I-1] Info on $domainname OK");
    $ua1->get_ok("http://localhost/domain/action?domainop=Update&domainname=$domainname&nsset=$nsset_1&nsset=$nsset_2&nsset=$nsset_3&registrant=$registrant&admin=$srid_ppp&tech=$srid_pp&pw=$pw", "Update domain $domainname - change admin from $srid_pp to $srid_ppp which is related to another organization");
    $ua1->content_contains("update '$domainname' failed", "[023-J-1] Update domain $domainname - change admin from $srid_pp to $srid_ppp which is related to another organization - shall fail");
    $ua1->content_contains("Domain [$domainname]: Legal contact [$srid_ppp] does not belong to organization [$srid_o]", "[023-J-1] Update domain $domainname - change admin from $srid_pp to $srid_ppp which is related to another organization - failed");
}

# Update and change registrant, to another organization, also admin and tech need to be updated to $srid_ppp which belongs to that org. as well
$ua1->get_ok("http://localhost/domain/action?domainop=Info&domainname=$domainname&objdetails=on", "Info on $domainname");
$ua1->content_contains("info '$domainname' succeeded", "[023-I-1] Info on $domainname OK");


###
# Applicant dataset arguments for a registrant change need to change to new values and a newer date
# 
$versionnumber = "2.0";
$acceptdate = $past . "T16:30:48Z";
$acceptname = "Tante Sofie Ny";
$appldataset_args = "versionnumber=$versionnumber&acceptdate=$acceptdate&acceptname=$acceptname&vfyapplicantdatasetupdate=on&objdetails=on";

my $admstr="";
if ($cf->{client}->{old_data_model}) {
    $admstr="&admin=$srid_ppp";
}	

$ua1->get_ok("http://localhost/domain/action?domainop=Update&domainname=$domainname&nsset=$nsset_1&nsset=$nsset_2&nsset=$nsset_3&registrant=$srid_o3$admstr&tech=$srid_r&pw=$pw&$appldataset_args", "Update domain $domainname - change registrant from $srid_o to to another organization $srid_o3 - shall be put in pending");
    
$ua1->content_contains("update '$domainname' succeeded", "[023-J-4, 023-JJ-2d]] Update domain $domainname - Update domain $domainname - change registrant from $srid_o to to another organization $srid_o3 OK");

#print $ua1->content;
#exit;

SKIP: {

    unless ($cf->{dnssec}->{enabled}) {
	skip "DNSSEC is disabled - all DNSSEC tests skipped";
    }

###############
# DNSSEC tests
#
# DNSSEC data arguments crafted especially for domain 'example.no'.
# - the domain need to exist to awoid being forced to generate keys for a domain every time the test is to be performed
# 
# 
#

my $domainname_dnssec     = 'norid-eppclient-dnssec-test.no';
my $domainname_dnssec_err = 'err-norid-eppclient-dnssec-test.no';

=head1 Test keys for the domain

The test keys are generated externally and copied into this test file.

=head2 KSK key file 1 

File name: Knorid-eppclient-dnssec-test.no.+008+09286.key
Contents:
; This is a key-signing key, keyid 9286, for norid-eppclient-dnssec-test.no.
; Created: 20140109125706 (Thu Jan  9 13:57:06 2014)
; Publish: 20140109125706 (Thu Jan  9 13:57:06 2014)
; Activate: 20140109125706 (Thu Jan  9 13:57:06 2014)
norid-eppclient-dnssec-test.no. IN DNSKEY 257 3 8 AwEAAclpg/bDh33n4FPd5niRxsN6RoPqc9YEQUaKVylHO+d1wpIPNqlQ 3YWjUPy8pSO7MF4UDc8dzlshCETxbxZRHLUv1T3Rr30VnWC+mhsf/Xgq fWVYdQ/kf0XlxUhF9D+mI6MNsHW8hz/atXtk4G9wCjwligPysFSD6Tie /B2UivMwYf/D2s1W5Px1hPM7oUEmJQq3baWqIEI6mGeuQBNYrwFGUZhg xWK2USGUE2/P2B4RdkZnrmZzqZ2sSjko5KRhuUlXRcrjLJtSEYaQOMk0 YnfES/2VPLntxzpyGwZVx+jftEAswVpG4wr+I4vkWLH72cRstyqnjqKx XqaMvm/ba4M=

=head2 KSK key file 2 

File name: Knorid-eppclient-dnssec-test.no.+008+22780.key
Contents:
; This is a key-signing key, keyid 22780, for norid-eppclient-dnssec-test.no.
; Created: 20140109125704 (Thu Jan  9 13:57:04 2014)
; Publish: 20140109125704 (Thu Jan  9 13:57:04 2014)
; Activate: 20140109125704 (Thu Jan  9 13:57:04 2014)
norid-eppclient-dnssec-test.no. IN DNSKEY 257 3 8 AwEAAdroS3LWi14j+IyTptKBuDZSVl+GUSv1/PeJRLrFxcaRa0eA4M4Y zj1ORt6QrXAbLtZgwqe7cQaTNog/Qmjl5N1OPmJXse2v3tnTA7hGxZh5 fUVvobdaj3bVD0DD0HGAevzjbgBjvo1nV88DWTgzcwMS4ESfBu71+spp LBpzDmNfU9qtzF+BKFv48Av5Ylt5kpNbF5pLfflbMgFUQXU8ppOcIEm4 e2OMDkQCRXiDaVlZEslrYs0cus7ksApcSXNkeSZTAsLuxTyLrr+QePUa U0YqDWqSCmTTsx7i3J6g3Sxg7fGdiEWYEnq6oR+cE3rxnBz9Zncj9hd5 lHhZHPoyIIk=


=head2 DS sets generated from the above two KSK keys

File name: dsset-norid-eppclient-dnssec-test.no.

Contents:
norid-eppclient-dnssec-test.no. IN DS 9286 8 1 A970213D5E33F0952DEF8C849DEADC6FFC7E3317
norid-eppclient-dnssec-test.no. IN DS 9286 8 2 92217DCCAF8C50EC0F8DF62B821A3A506ED9163A589D839FEA4119D0 2A6C5EEA
norid-eppclient-dnssec-test.no. IN DS 22780 8 1 D532A75D5254828A00841B206E71DBA723E9337C
norid-eppclient-dnssec-test.no. IN DS 22780 8 2 A1B78EA51E9A8A2CFB47BB2061886C8603C8EA6728DABE43EB1EF865 5095A877

The key data is used as test data below.

=cut

my $keytag_err  = 70000;
my $keytag1     = 9286;
my $keytag2     = 22780;

my $alg_err      = 70;
my $alg_mismatch = 10;
my $alg          = 8;

my $digtype_err = 80;
my $digtype1    = 1;
my $digtype2    = 2;

my $digest_err   = lc('A970213D5E33F0952DEF8C849DEADC6FFC7E331');
my $digest_err1c = lc('A970000000000000000000000000000000000007');

my $digest1_1  = lc('A970213D5E33F0952DEF8C849DEADC6FFC7E3317');
my $digest1_2  = lc('92217DCCAF8C50EC0F8DF62B821A3A506ED9163A589D839FEA4119D02A6C5EEA');

my $digest2_1  = lc('D532A75D5254828A00841B206E71DBA723E9337C');
my $digest2_2  = lc('A1B78EA51E9A8A2CFB47BB2061886C8603C8EA6728DABE43EB1EF8655095A877');

my $key_flags_err = 256;
my $key_flags     = 257;

my $key_protocol_err = 4;
my $key_protocol     = 3;

my $pubkey_err = uri_escape('ERRORclpg/bDh33n4FPd5niRxsN6RoPqc9YEQUaKVylHO+d1wpIPNqlQ 3YWjUPy8pSO7MF4UDc8dzlshCETxbxZRHLUv1T3Rr30VnWC+mhsf/Xgq fWVYdQ/kf0XlxUhF9D+mI6MNsHW8hz/atXtk4G9wCjwligPysFSD6Tie /B2UivMwYf/D2s1W5Px1hPM7oUEmJQq3baWqIEI6mGeuQBNYrwFGUZhg xWK2USGUE2/P2B4RdkZnrmZzqZ2sSjko5KRhuUlXRcrjLJtSEYaQOMk0 YnfES/2VPLntxzpyGwZVx+jftEAswVpG4wr+I4vkWLH72cRstyqnjqKx XqaMvm/ba4M=');

my $pubkey1    = uri_escape('AwEAAclpg/bDh33n4FPd5niRxsN6RoPqc9YEQUaKVylHO+d1wpIPNqlQ 3YWjUPy8pSO7MF4UDc8dzlshCETxbxZRHLUv1T3Rr30VnWC+mhsf/Xgq fWVYdQ/kf0XlxUhF9D+mI6MNsHW8hz/atXtk4G9wCjwligPysFSD6Tie /B2UivMwYf/D2s1W5Px1hPM7oUEmJQq3baWqIEI6mGeuQBNYrwFGUZhg xWK2USGUE2/P2B4RdkZnrmZzqZ2sSjko5KRhuUlXRcrjLJtSEYaQOMk0 YnfES/2VPLntxzpyGwZVx+jftEAswVpG4wr+I4vkWLH72cRstyqnjqKx XqaMvm/ba4M=');

my $pubkey2    =  uri_escape('AwEAAdroS3LWi14j+IyTptKBuDZSVl+GUSv1/PeJRLrFxcaRa0eA4M4Y zj1ORt6QrXAbLtZgwqe7cQaTNog/Qmjl5N1OPmJXse2v3tnTA7hGxZh5 fUVvobdaj3bVD0DD0HGAevzjbgBjvo1nV88DWTgzcwMS4ESfBu71+spp LBpzDmNfU9qtzF+BKFv48Av5Ylt5kpNbF5pLfflbMgFUQXU8ppOcIEm4 e2OMDkQCRXiDaVlZEslrYs0cus7ksApcSXNkeSZTAsLuxTyLrr+QePUa U0YqDWqSCmTTsx7i3J6g3Sxg7fGdiEWYEnq6oR+cE3rxnBz9Zncj9hd5 lHhZHPoyIIk=');

# OK stuff
my $dnssec_ds1_1  = "keyTag=$keytag1&alg=$alg&digestType=$digtype1&digest=$digest1_1";
my $dnssec_ds1_2  = "keyTag=$keytag1&alg=$alg&digestType=$digtype2&digest=$digest1_2";
my $dnssec_ds2_1  = "keyTag=$keytag2&alg=$alg&digestType=$digtype1&digest=$digest2_1";
my $dnssec_ds2_2  = "keyTag=$keytag2&alg=$alg&digestType=$digtype2&digest=$digest2_2";

my $dnssec_ds_args1   = "$dnssec_ds1_1&$dnssec_ds1_2";
my $dnssec_ds_args2   = "$dnssec_ds2_1&$dnssec_ds2_2";
my $dnssec_ds_args1_2 = "$dnssec_ds_args1&$dnssec_ds_args2";  

my $dnssec_key_args1   = "key_keyTag=$keytag1&key_alg=$alg&key_flags=$key_flags&key_protocol=$key_protocol&key_pubKey=$pubkey1";
my $dnssec_key_args2   = "key_keyTag=$keytag2&key_alg=$alg&key_flags=$key_flags&key_protocol=$key_protocol&key_pubKey=$pubkey2";
my $dnssec_key_args1_2 = "$dnssec_key_args1&$dnssec_key_args2";

###
# Error stuff
my $dnssec_ds_args_err_keytag  = "keyTag=$keytag_err&alg=$alg&digestType=$digtype1&digest=$digest1_1";

# error in alg
my $dnssec_ds_args_err_alg     = "keyTag=$keytag1&alg=$alg_err&digestType=$digtype1&digest=$digest1_1";

# error in digType
my $dnssec_ds_args_err_digtype = "keyTag=$keytag1&alg=$alg&digestType=$digtype_err&digest=$digest1_1";

# error in digest
my $dnssec_ds_args_err_digest   = "keyTag=$keytag1&alg=$alg&digestType=$digtype1&digest=$digest_err";
my $dnssec_ds_args_err_digest1c = "keyTag=$keytag1&alg=$alg&digestType=$digtype1&digest=$digest_err1c";

# error in key_keytag
my $dnssec_key_args_err_keytag = "key_keyTag=$keytag_err&key_alg=$alg&key_flags=$key_flags&key_protocol=$key_protocol&key_pubKey=$pubkey1";

# error in key_key_alg
my $dnssec_key_args_err_alg      = "key_keyTag=$keytag1&key_alg=$alg_err&key_flags=$key_flags&key_protocol=$key_protocol&key_pubKey=$pubkey1";
my $dnssec_key_args_mismatch_alg = "key_keyTag=$keytag1&key_alg=$alg_mismatch&key_flags=$key_flags&key_protocol=$key_protocol&key_pubKey=$pubkey1";


## Error DNSSEC stuff
#

#
# 1) DS only, with various errors in DS parameters:
#

# Create with illegal keytag
$ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=$domainname_dnssec&nsset=$nsset_1&nsset=$nsset_2&registrant=$registrant$adminstr&tech=$tech&pw=$pw&$appldataset_args&$dnssec_ds_args_err_keytag", "Create DNSSEC domain $domainname w/DNSSEC DS with illegal keytag, shall be rejected ");
$ua1->content_contains("keyTag must be an integer in the range 0-65535: '$keytag_err'", "[023-J-1b] Create $domainname_dnssec w/DNSSEC DS with illegal keytag, rejected");

# Create with illegal alg
$ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=$domainname_dnssec&nsset=$nsset_1&nsset=$nsset_2&registrant=$registrant$adminstr&tech=$tech&pw=$pw&$appldataset_args&$dnssec_ds_args_err_alg", "Create DNSSEC domain $domainname w/DNSSEC DS with illegal alg, shall be rejected ");
$ua1->content_contains("DS alg is not among the permitted algorithm values:", "[023-J-1b] Create $domainname_dnssec w/DNSSEC DS with illegal alg, rejected");

# Create with illegal digType
$ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=$domainname_dnssec&nsset=$nsset_1&nsset=$nsset_2&registrant=$registrant$adminstr&tech=$tech&pw=$pw&$appldataset_args&$dnssec_ds_args_err_digtype", "Create DNSSEC domain $domainname w/DNSSEC DS with illegal digType, shall be rejected ");
#print $ua1->content;
$ua1->content_contains("DS digestType is not among the permitted types:", "[023-J-1b] Create $domainname_dnssec w/DNSSEC DS with illegal digType, rejected");

# Create with illegal digest (too short, locally rejected by the client)
# (Note: a digest with correct length and correct characters, but invalid hash, can only be validated if the key is also passed, this is tested below.)
$ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=$domainname_dnssec&nsset=$nsset_1&nsset=$nsset_2&registrant=$registrant$adminstr&tech=$tech&pw=$pw&$appldataset_args&$dnssec_ds_args_err_digest", "Create DNSSEC domain $domainname w/DNSSEC DS with illegal (tto short) digest, shall be rejected ");
$ua1->content_contains("DS invalid digest length ", "[023-J-1b] Create $domainname_dnssec w/DNSSEC DS with illegal (too short) digest, rejected");

#
# 2) DS+keys and error situations
#

# Create with illegal DS digest for its key, its length is correct, but the hash is not a correct hash of the key. The policy checks should detect this and reject.
$ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=$domainname_dnssec&nsset=$nsset_1&nsset=$nsset_2&registrant=$registrant$adminstr&tech=$tech&pw=$pw&$appldataset_args&$dnssec_ds_args_err_digest1c&$dnssec_key_args1", "Create DNSSEC domain $domainname w/DNSSEC DS+key with illegal digest, shall be rejected");
$ua1->content_contains("create '$domainname_dnssec' failed.");
$ua1->content_contains("Invalid digest [$digest_err1c]. Mismatch with key.", "[023-J-1b] Create $domainname_dnssec w/DNSSEC DS+key with illegal digest, rejected");
    
# Create with illegal another domain name than matches the key/DS
$ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=$domainname_dnssec_err&nsset=$nsset_1&nsset=$nsset_2&registrant=$registrant$adminstr&tech=$tech&pw=$pw&$appldataset_args&$dnssec_ds_args1&$dnssec_key_args1", " Create with illegal another domain name than matches the key/DS, shall fail ");
$ua1->content_contains("create '$domainname_dnssec_err' failed", "[023-J-1] Create DNSSEC $domainname_dnssec_err failed");
$ua1->content_contains("Invalid digest [$digest1_2]. Mismatch with key.", "[023-J-1b] Create $domainname_dnssec_err w/DNSSEC DS+key but key is for another domain, rejected");

# Create with correct DS, but specified keyTag is illegal (out of range)
$ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=$domainname_dnssec&nsset=$nsset_1&nsset=$nsset_2&registrant=$registrant$adminstr&tech=$tech&pw=$pw&$appldataset_args&$dnssec_ds_args1&$dnssec_key_args_err_keytag", "Create DNSSEC domain $domainname w/DNSSEC DS+key with illegal keyTag, shall be rejected");
$ua1->content_contains("KEY keyTag must be an integer in the range 0-65535: '$keytag_err'", "[023-J-1b] Create $domainname_dnssec w/DNSSEC DS+key with illegal keyTag, rejected");

# Create with correct DS, but specified keyAlg is illegal
$ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=$domainname_dnssec&nsset=$nsset_1&nsset=$nsset_2&registrant=$registrant$adminstr&tech=$tech&pw=$pw&$appldataset_args&$dnssec_ds_args1&$dnssec_key_args_err_alg", "Create DNSSEC domain $domainname w/DNSSEC DS+key with illegal keyAlg, shall be rejected");
$ua1->content_contains("keyTag $keytag1:  KEY alg is not among the permitted algorithm values: $alg_err, permitted are: ", "[023-J-1b] Create $domainname_dnssec w/DNSSEC DS+key with illegal keyAlg, rejected");

# Create with a correct DS and key, and specified keyAlg is legal but different from the ones for the DSs
$ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=$domainname_dnssec&nsset=$nsset_1&nsset=$nsset_2&registrant=$registrant$adminstr&tech=$tech&pw=$pw&$appldataset_args&$dnssec_ds_args1&$dnssec_key_args_mismatch_alg", "Create DNSSEC domain $domainname w/DNSSEC DS+key with mismatched keyAlg, shall be rejected");
$ua1->content_contains("keyTag $keytag1:  KEY cannot be used because there are no matching DS record for tag: $keytag1 and alg: $alg_mismatch", "[023-J-1b] Create $domainname_dnssec w/DNSSEC DS+key with mismatched keyAlg, rejected");

# Create domain with DNSSEC-key only parameters, rejected because no DSs are specified
$ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=$domainname_dnssec&nsset=$nsset_1&nsset=$nsset_2&registrant=$registrant$adminstr&tech=$tech&pw=$pw&$appldataset_args&$dnssec_key_args1", "Create domain $domainname_dnssec, no DS-only key, shall be rejected");
$ua1->content_contains("keyTag $keytag1:  KEY cannot be used because there are no matching DS record for tag: $keytag1",               "[023-J-1b] Create $domainname_dnssec w/DNSSEC, key only, no DS, rejected, no DS for this tag");
$ua1->content_contains("keyTag $keytag1:  KEY cannot be used because there are no matching DS record for tag: $keytag1 and alg: $alg", "[023-J-1b] Create $domainname_dnssec w/DNSSEC, key only, no DS, rejected, no DS for this tag and alg");


## OK DNSSEC stuff
#
# Create domain with DNSSEC DS-parameters
$ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=$domainname_dnssec&nsset=$nsset_1&nsset=$nsset_2&registrant=$registrant$adminstr&tech=$tech&pw=$pw&$appldataset_args&$dnssec_ds_args1", "Create DNSSEC domain $domainname, shall succeed ");
$ua1->content_contains("create '$domainname_dnssec' succeeded", "[023-J-1b] Create $domainname_dnssec w/DNSSEC DS succeeded");

# Now create domain with DNSSEC-DS-and-key parameters, the domain shall exist, but be rejected due to existing domain, not because of illegal parameters
$ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=$domainname_dnssec&nsset=$nsset_1&nsset=$nsset_2&registrant=$registrant$adminstr&tech=$tech&pw=$pw&$appldataset_args&$dnssec_ds_args1&$dnssec_key_args1", "Create same domain $domainname once more, shall fail ");
$ua1->content_contains("create '$domainname_dnssec' failed", "[023-J-1] Create $domainname_dnssec w/DNSSEC DS and key - already exists");
$ua1->content_contains("Domain with name [$domainname_dnssec] already exists in the database", "[023-J-1b] Create $domainname_dnssec w/DNSSEC DS and key args failed - already exists in the registry");

###
# Update most attributes: host and AuthInfo, admin and tech on a DNSSEC enabled domain
$pw = 'Newpw12345';
$ua1->get_ok("http://localhost/domain/action?domainop=Update&domainname=$domainname_dnssec&nsset=$nsset_1&nsset=$nsset_2&nsset=$nsset_3&registrant=$registrant$adminstr&tech=$srid_r&pw=$pw&$dnssec_ds_args1", "Update domain $domainname_dnssec AuthInfo, add a third host, change admin and tech contacts from $srid_p to $srid_pp");
$ua1->content_contains("update '$domainname_dnssec' succeeded", "[023-J-4b] Update domain $domainname_dnssec w/DNSSSEC -  AuthInfo, add a third host, change admin and tech contacts from $srid_p to $srid_pp");

# Update authInfo and DNSSEC DS data from $dnssec_ds_args1 to $dnssec_ds_args2, shall be rejected because no verify
$pw = 'NewpwXX12345';
$ua1->get_ok("http://localhost/domain/action?domainop=Update&domainname=$domainname_dnssec&nsset=$nsset_1&nsset=$nsset_2&nsset=$nsset_3&registrant=$registrant$adminstr&tech=$tech&pw=$pw&$dnssec_ds_args2", "Update domain $domainname_dnssec AuthInfo and DNSSEC DS - shall be rejected due to missing verify");
$ua1->content_contains("Update of DS or key data requires a checked 'verify update' field", "[023-J-4b] Update domain $domainname_dnssec w/DNSSSEC -  AuthInfo and DS failed, no verify.");

# Update DNSSEC DS data from $dnssec_ds_args2 to $dnssec_ds_args1, shall be accepted
$ua1->get_ok("http://localhost/domain/action?domainop=Update&domainname=$domainname_dnssec&nsset=$nsset_1&nsset=$nsset_2&nsset=$nsset_3&registrant=$registrant$adminstr&tech=$tech&pw=$pw&$dnssec_ds_args1&vfydnssecupdate=on", "Update domain $domainname_dnssec and swap DNSSEC DS record");
$ua1->content_contains("update '$domainname_dnssec' succeeded", "[023-J-4b] Update domain $domainname_dnssec w/DNSSSEC - swap DS record, success.");

# Update DNSSEC DS data from $dnssec_ds_args1 to $dnssec_ds_args1_2, thus adding a second DS, and also add key $dnssec_key_args1 which is the key for 1_1
$ua1->get_ok("http://localhost/domain/action?domainop=Update&domainname=$domainname_dnssec&nsset=$nsset_1&nsset=$nsset_2&nsset=$nsset_3&registrant=$registrant$adminstr&tech=$tech&pw=$pw&$dnssec_ds_args1_2&vfydnssecupdate=on&$dnssec_key_args1", "Update domain $domainname_dnssec, DNSSEC add a second DS and add key for one of the DSs");
$ua1->content_contains("update '$domainname_dnssec' succeeded", "[023-J-4b] Update domain $domainname_dnssec w/DNSSSEC - add a second DS and add key for one of the DSs, success");

# Update DNSSEC DS data and add also the second key
$ua1->get_ok("http://localhost/domain/action?domainop=Update&domainname=$domainname_dnssec&nsset=$nsset_1&nsset=$nsset_2&nsset=$nsset_3&registrant=$registrant$adminstr&tech=$tech&pw=$pw&$dnssec_ds_args1_2&vfydnssecupdate=on&$dnssec_key_args1_2", "Update domain $domainname_dnssec, DNSSEC DS and add also second key");
$ua1->content_contains("update '$domainname_dnssec' succeeded", "[023-J-4b] Update domain $domainname_dnssec w/DNSSSEC - DS and add also second key");

# Update DNSSEC DS data and add remove all DNSSEC data using the dnssecdeleteall parameter
$ua1->get_ok("http://localhost/domain/action?domainop=Update&domainname=$domainname_dnssec&nsset=$nsset_1&nsset=$nsset_2&nsset=$nsset_3&registrant=$registrant$adminstr&tech=$tech&pw=$pw&$dnssec_ds_args1_2&vfydnssecupdate=on&$dnssec_key_args1_2&dnssecdeleteall=on", "Update domain $domainname_dnssec, DNSSEC DS - remove all DNSSEC data using the dnssecdeleteall parameter");
$ua1->content_contains("update '$domainname_dnssec' succeeded", "[023-J-4b] Update domain $domainname_dnssec w/DNSSSEC - remove all DNSSEC data using the dnssecdeleteall parameter - success");

} # END SKIP


##
# Check

# missing name
$ua1->get_ok("http://localhost/domain/action?domainop=Check&domainname=", "Check on missing domainname");
$ua1->content_contains("'check' requires a name", "[023-I-4] Check on missing domainname failed");

# invalid name
my $illname = "ille_gal.no";
$ua1->get_ok("http://localhost/domain/action?domainop=Check&domainname=$illname", "Check on invalid domainname");
$ua1->content_contains("check: illegal domainname: '$illname'", "[023-I-4] Check on invalid domainname failed");

# nonexisting name
my $nxname = "nonexisting-$sfx-nxt.no";
$ua1->get_ok("http://localhost/domain/action?domainop=Check&domainname=$nxname", "Check on non-existing domainname");
$ua1->content_contains("check '$nxname' succeeded", "[023-I-4] Check on non-existing $nxname performed");
$ua1->content_contains("Domain does not exist.", "[023-I-4] Check on non-existing domainname failed");

# ok name
$ua1->get_ok("http://localhost/domain/action?domainop=Check&domainname=$domainname", "Check on $domainname");
$ua1->content_contains("check '$domainname' succeeded", "[023-I-4] Check on $domainname OK");


###
# Info

# domain name empty
$ua1->get_ok("http://localhost/domain/action?domainop=Info&domainname=", "Info on empty domainname - shall fail");
$ua1->content_contains("'info' requires a name", "[023-I-1] Info on empty domainname - failed");

# domain name non-existing
$ua1->get_ok("http://localhost/domain/action?domainop=Info&domainname=$nxname", "Info on non-existing domainname $nxname - shall fail");
$ua1->content_contains("info '$nxname' failed", "[023-I-1] Info on non-existing domainname $nxname - failed");

# domain under other registrar
$ua1->get_ok("http://localhost/domain/action?domainop=Info&domainname=$domainnameb&objdetails=on", "Info on $domainnameb - under other registrar");
$ua1->content_contains("info '$domainnameb' succeeded", "[023-I-1] Info on $domainnameb - under other registrar OK");

# some attributes should not be visible, esp. authInfo -  we did set it on the Create above, so check that it is empty
$ua1->content_contains("[hidden, not sponsor]", "[023-I-1] Info on $domainnameb - under other registrar - hidden no sponsor is stated");
my $opw = $ua1->form_number('1')->value('pw');
ok(!$opw,  "Info on $domainnameb - under other registrar - authInfo field is empty - no sponsor");


# OK domain
$ua1->get_ok("http://localhost/domain/action?domainop=Info&domainname=$domainname&objdetails=on", "Info on $domainname");
$ua1->content_contains("info '$domainname' succeeded", "[023-I-1] Info on $domainname OK");

# verify proper sponsorship
my $c = $ua1->content(format => 'text');
$regexp = qr/(Client ID:)\s*(reg\d+)/;
my $regid;
if ($c =~ $regexp) {
    $regid = $2;
}
ok ($regid && $regid eq $rega, "[023-B-1] Correct domain object sponsorship verified");


###
# Renew the domain created above
# The current expiry is set to one year ahead during create
# 
my @now = gmtime;
my $expiry = sprintf "%04d-%02d-%02d\n", $now[5]+1901, $now[4]+1, $now[3];
my $period = 6;  # months
#print "expiry: $expiry\n";

# First a renew with illegal period
my $illperiod = -1;
$ua1->get_ok("http://localhost/domain/action?domainop=Renew&domainname=$domainname&duration=$illperiod&vfyrenew=on&curexpiry=$expiry", "Renew $domainname for $illperiod months shall fail");
$ua1->content_contains("Invalid duration", "[023-J-7] Renew $domainname for $illperiod months failed");

$illperiod = 13;
$ua1->get_ok("http://localhost/domain/action?domainop=Renew&domainname=$domainname&duration=$illperiod&vfyrenew=on&curexpiry=$expiry", "Renew $domainname for $illperiod months shall fail");
$ua1->content_contains("Invalid duration", "[023-J-7] Renew $domainname for $illperiod months failed");

# Renew with a sensible period, not verified
$ua1->get_ok("http://localhost/domain/action?domainop=Renew&domainname=$domainname&duration=$period&vfyrenew=&curexpiry=$expiry", "Renew $domainname for $period months shall not be accepted - not verified ");
$ua1->content_contains("Renew requires a checked 'verify' field", "[023-J-7] Renew $domainname for $period months was not accepted - not verified");


# Renew without a domain name, verified
$ua1->get_ok("http://localhost/domain/action?domainop=Renew&domainname=&duration=$period&vfyrenew=on&curexpiry=$expiry", "Renew without a domainname shall be rejected");
$ua1->content_contains("'renew' requires a name", "[023-J-7] Renew without a domainname was rejected");


# Renew with a sensible period, verified, should fail because we're outside the permitted renew window
$ua1->get_ok("http://localhost/domain/action?domainop=Renew&domainname=$domainname&duration=$period&vfyrenew=on&curexpiry=$expiry", "Renew $domainname for $period months shall be rejected");
$ua1->content_contains("renew '$domainname' failed", "[023-J-7] Renew $domainname for $period months was rejected");
$ua1->content_contains("Domain [$domainname] is not renewable before [", "[023-J-7] Renew $domainname for $period months was rejected because outside of renew window");
 

# Renew a domain owned by other sponsor, verified, shall fail
$ua1->get_ok("http://localhost/domain/action?domainop=Renew&domainname=$domainnameb&duration=$period&vfyrenew=on&curexpiry=$expiry", "Renew $domainname under other sponsor for $period months shall be rejected");
#print $ua1->content;
$ua1->content_contains("renew '$domainnameb' failed", "[023-J-7] Renew $domainnameb under other sponsor for $period months was rejected");

### Create a withdraw domain and withdraw it. 
# 
$ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=$domainname2&nsset=$nsset_1&nsset=$nsset_2&registrant=$registrant$adminstr&tech=$tech&pw=$pw&$appldataset_args", "Create withdraw test domain");
$ua1->content_contains("create '$domainname2' succeeded", "[023-J-1] Create $domainname2 with the same ns and contacts as for domain $domainname OK");


## withdraw a domain under other sponsor, shall fail
$ua1->get_ok("http://localhost/domain/action?domainop=Withdraw&domainname=$domainnameb&vfywithdraw=on", "Withdraw $domainnameb - a domain under other sponsor - shall fail");
#print $ua1->content;
$ua1->content_contains("withdraw '$domainnameb' failed", "[023-J-13] Withdraw $domainnameb, a domain under other sponsor - failed");

## withdraw it, not verified
$ua1->get_ok("http://localhost/domain/action?domainop=Withdraw&domainname=$domainname2&vfywithdraw=", "Withdraw $domainname2, not verified shall fail");
$ua1->content_contains("Withdraw requires a checked 'verify' field", "[023-J-13] Withdraw $domainname2, not verified failed");

## withdraw it, verified
$ua1->get_ok("http://localhost/domain/action?domainop=Withdraw&domainname=$domainname2&vfywithdraw=on", "Withdraw $domainname2, verified");
$ua1->content_contains("withdraw '$domainname2' succeeded", "[023-J-13] Withdraw $domainname2 OK");

my (@rreg, @radmin, @rtech, @r0reg, @r0admin, @r0tech, @r0nsset, %r0);

if ($ua0) {
    # collect the handles under reg0
    $ua0->get_ok("http://localhost/domain/action?domainop=Info&domainname=$domainname2", "Info on reg0 domain $domainname2 ...");
    $ua0->content_contains("info '$domainname2' succeeded", "Info on reg0 domain $domainname2 succeeded");

    # Collect all the contactIDs on the reg0 account
    push @r0reg  , $ua0->form_number('1')->value('registrant');
    push @r0admin, $ua0->form_number('1')->param('admin');
    push @r0tech , $ua0->form_number('1')->param('tech');

    @r0nsset = $ua0->form_number('1')->param('nsset');
    foreach my $n (@r0nsset) {
	$r0{$n} = 1;
    } 
}

###
# Reclaim the withdrawn domain. Two variants:
# 1) Normal transfer with empty AuthInfo
# 2) Request token, using simsalabim


# 1) Normal transfer with empty AuthInfo
$ua1->get_ok("http://localhost/domain/action?domainop=Transfer_wAuthInfo&domainname=$domainname2&vfytransferwauthinfo=on&pw=", "Transfer Domain from reg0 - empty authInfo shall be OK");
$ua1->content_contains("transfer_execute '$domainname2' succeeded", "[023-K-?] Transfer Domain from reg0 - empty authInfo shall be OK");


## withdraw it again, verified
$ua1->get_ok("http://localhost/domain/action?domainop=Withdraw&domainname=$domainname2&vfywithdraw=on", "Withdraw $domainname2, verified");
$ua1->content_contains("withdraw '$domainname2' succeeded", "[023-J-13] Withdraw $domainname2 OK");


# 2) Request token, using simsalabim
# We simulate that we don't know the authInfo.
# The token based variant must be used to reclaim it, but that is not good for automated testing, but
# we use the token based variant with simsalabim as long as this development trick works.
# First Request a token to start the transfer, then do a Transferwtoken
my $token='simsalabim';
$ua1->get_ok("http://localhost/domain/action?domainop=RequestToken&domainname=$domainname2", "Request a one time token ...");
$ua1->content_contains("transfer '$domainname2' succeeded", "[023-J-16] Transfer request of domain $domainname2 succeeded" );

## TODO: Error in registry, exception closed connection. Tries to send to legal-c email address, which is not registered.


$ua1->get_ok("http://localhost/domain/action?domainop=Transfer_wToken&domainname=$domainname2&token=$token&vfytransferwtoken=on", "Transfer Domain back from REG0 ...");
$ua1->content_contains("transfer_execute '$domainname2' succeeded", "[023-K-4] Transfer with token of domain $domainname2 succeeded");




# Collect all the contactIDs after reclaim
push @rreg  , $ua1->form_number('1')->value('registrant');
push @radmin, $ua1->form_number('1')->param('admin');
push @rtech , $ua1->form_number('1')->param('tech');


####
# Delete the Domains created above

my $deletefromdns      = $today;
my $deletefromregistry = $future;

# delete, no domain name
$ua1->get_ok("http://localhost/domain/action?domainop=Delete&domainname=&vfydelete=on", "Delete without domainname shall fail");
$ua1->content_contains("'delete' requires a name", "[023-J-11] Delete without domainname failed");

# delete, not verified
$ua1->get_ok("http://localhost/domain/action?domainop=Delete&domainname=$domainname&vfydelete=", "Delete '$domainname', not verified shall fail");
$ua1->content_contains("Delete requires a checked 'verify' field", "[023-J-11] Delete '$domainname', not verified failed");


# Delete from DNS only
$ua1->get_ok("http://localhost/domain/action?domainop=Delete&domainname=$domainname&vfydelete=on&deletefromdns=$deletefromdns", "Delete '$domainname', 'Delete from DNS' only, shall fail");
$ua1->content_contains("Delete with dates requires both of the dates", "[023-J-11] Delete '$domainname', 'Delete from DNS' only, failed");
$ua1->content_contains("Delete requires a valid 'Delete from Registry' date", "[023-J-11] Delete '$domainname', 'Delete from DNS' only, failed");

# Delete from Registry only
$ua1->get_ok("http://localhost/domain/action?domainop=Delete&domainname=$domainname&vfydelete=on&deletefromregistry=$deletefromregistry", "Delete '$domainname', 'Delete from Registry' only, shall fail");
$ua1->content_contains("Delete with dates requires both of the dates", "[023-J-11] Delete '$domainname', 'Delete from Registry' only, failed");
$ua1->content_contains("Delete requires a valid 'Delete from DNS' date", "[023-J-11] Delete '$domainname', 'Delete from Registry' only, failed");


# Delete from DNS and Registry, but invalid dates
my $illdeletefromdns      = '2009-09-32';
my $illdeletefromregistry = '2009-00-28';
$ua1->get_ok("http://localhost/domain/action?domainop=Delete&domainname=$domainname&vfydelete=on&deletefromdns=$illdeletefromdns&deletefromregistry=$illdeletefromregistry", "Delete '$domainname' with invalid dates shall fail");
$ua1->content_contains("Delete requires a valid 'Delete from DNS' date", "[023-J-11] Delete '$domainname' with invalid dates failed");
$ua1->content_contains("Delete requires a valid 'Delete from Registry' date", "[023-J-11] Delete '$domainname' with invalid dates failed");

$deletefromdns      = $past;
$deletefromregistry = $past;
#Delete IDN failed, both past dates
$ua1->get_ok(encode("UTF-8", decode("ISO8859-1","http://localhost/domain/action?domainop=Delete&domainname=$dn_idn&vfydelete=on&deletefromdns=$deletefromdns&deletefromregistry=$deletefromregistry")), "Delete IDN '$dn_utf' with both past dates - shall fail");
#print $ua1->content;
$ua1->content_contains("'Delete from DNS' must be &gt;= todays date '$today'", "[023-J-11] Delete IDN '$dn_utf' with both past dates - failed");

$deletefromdns      = $today;
$deletefromregistry = $past;
#Delete IDN failed, one past date
$ua1->get_ok(encode("UTF-8", decode("ISO8859-1","http://localhost/domain/action?domainop=Delete&domainname=$dn_idn&vfydelete=on&deletefromdns=$deletefromdns&deletefromregistry=$deletefromregistry")), "Delete IDN '$dn_utf' with one past date shall fail");
$ua1->content_contains("'Delete from DNS' must be &lt;= 'Delete from Registry'", "[023-J-11] Delete IDN '$dn_utf' with one past date failed");

#Delete IDN ok, normal dates
$deletefromdns      = $tomorrow;
$deletefromregistry = $future;
$ua1->get_ok(encode("UTF-8", decode("ISO8859-1","http://localhost/domain/action?domainop=Delete&domainname=$dn_idn&vfydelete=on&deletefromdns=$deletefromdns&deletefromregistry=$deletefromregistry")), "Delete IDN '$dn_utf' normal operation, with dates");
#print $ua1->content;
#print "sent: http://localhost/domain/action?domainop=Delete&domainname=$dn_idn&vfydelete=on&deletefromdns=$deletefromdns&deletefromregistry=$deletefromregistry\n";
$ua1->content_contains("delete '$dn_idn' succeeded", "[023-J-11] Delete IDN '$dn_utf', normal operation with dates OK");

# Delete with both dates tomorrow
$deletefromdns      = $tomorrow;
$deletefromregistry = $tomorrow;
$ua1->get_ok("http://localhost/domain/action?domainop=Delete&domainname=$domainname&vfydelete=on&deletefromdns=$deletefromdns&deletefromregistry=$deletefromregistry", "Delete '$domainname' with both dates tomorrow");
$ua1->content_contains("delete '$domainname' succeeded", "[023-J-11] Delete '$domainname' with both dates tomorrow OK");


# Delete with both dates today
$deletefromdns      = $today;
$deletefromregistry = $today;
$ua1->get_ok("http://localhost/domain/action?domainop=Delete&domainname=$domainname2&vfydelete=on&deletefromdns=$deletefromdns&deletefromregistry=$deletefromregistry", "Delete '$domainname' with both dates today");
$ua1->content_contains("delete '$domainname2' succeeded", "[023-J-11] Delete '$domainname' with both dates today OK");

# Delete other OK
$ua2->get_ok("http://localhost/domain/action?domainop=Delete&domainname=$domainnameb&vfydelete=on", "Delete '$domainnameb' normal operation");
$ua2->content_contains("delete '$domainnameb' succeeded", "[023-J-11] Delete '$domainnameb' normal operation OK");

### Try to remove the Organization contact from the Person contact
$ua1->get_ok("http://localhost/contact/action?contactop=Info&srid=$srid_p", "Info on $srid_p");
$ua1->content_contains("info '$srid_p' succeeded", "Info on $srid_p ok");
$ua1->get_ok("http://localhost/contact/action?contactop=Update&contactname=$contactname&type=$type&cc=$cc&pc=$pc&city=$city&email=$email&srid=$srid_p&organization=", "Try to remove the Organization contact: $srid_o from the Person contact: $srid_p, shall fail, in use");
$ua1->content_contains("update '$srid_p' failed", "[023-J-6] Try to remove the Organization contact: $srid_o from the Person contact: $srid_p, failed, OK");



### Try to update the $srid_o3 org with another orgnumber - shall be rejected!
$ua1->get_ok("http://localhost/contact/action?contactop=Info&srid=$srid_o3", "Info on $srid_o3");
$ua1->content_contains("info '$srid_o3' succeeded", "Info on $srid_o3 ok");
my $orgno4 = $t->{orgnoa3};;
my $identityvalue_4 = sprintf("%09d", $orgno4);
$ua1->get_ok("http://localhost/contact/action?contactop=Update&srid=$srid_o3&contactname=".uri_escape($contactname_1)."&type=$type_1&cc=$cc_1&pc=$pc_1&city=$city_1&email=$email_1&identitytype=$identitytype_1&identityvalue=$identityvalue_4", "Update owner organization $srid_o3 under $rega with a new orgnumber - shall be rejected");
$ua1->content_contains("update '$srid_o3' failed", "[023-J-4] Update owner organization $srid_o3 under $rega with a new orgnumber - failed");

$ua1->content_contains("Protocol error. Updating an identity of type organizationNumber is not supported", "[023-J-4] Update owner organization $srid_o3 under $rega with a new orgnumber - was rejected");


####
# Delete the Contacts created above
#
$ua1->get_ok("http://localhost/contact/action?contactop=Delete&srid=$srid_o", "Delete '$srid_o'...");
$ua1->content_contains("delete '$srid_o' failed", "[023-J-13] Delete '$srid_o' failed, is used because of withdraw object cloning ..OK");
my %hd;
foreach my $s ($srid_p, $srid_pp, $srid_ppp, $srid_o3, @radmin, @rtech) {
    next if $hd{$s};   # Skip if duplicate IDs
    $ua1->get_ok("http://localhost/contact/action?contactop=Delete&srid=$s", "Delete '$s' (under $rega)");
    $regexp = qr/(delete '$s' failed.)/;
    if ($ua1->content =~ $regexp) {
	$ua1->content_contains("Object [$s] is in use and cannot be deleted.", "Delete '$s' contact not done, in use");
    } else {
	$ua1->content_contains("delete '$s' succeeded", "Delete '$s' contact done");
    }
    $hd{$s} = 1;
}

foreach my $s ($srid_p2, $srid_o2) {
    $ua2->get_ok("http://localhost/contact/action?contactop=Delete&srid=$s", "Delete '$s' (under $regb)");
    $regexp = qr/(delete '$s' failed.)/;
    if ($ua2->content =~ $regexp) {
	$ua2->content_contains("Object [$s] is in use and cannot be deleted.", "Delete '$s' contact not done, in use");
    } else {
	$ua2->content_contains("delete '$s' succeeded", "Delete '$s' contact done");
    }
}

##
# Cleanup other objects 
foreach my $h ($hostname_1, $hostname_2, $hostname_3) {

    $ua1->get_ok("http://localhost/host/action/?hostop=Delete&hostname=$h", "Delete $rega host $h");

    $regexp = qr/(delete '$h' failed.)/;
    if ($ua1->content =~ $regexp) {
	$ua1->content_contains("delete '$h' failed.", "[023-J-12] Deleted $rega host '$h' not done, in use");
    } else {
	$ua1->content_contains("delete '$h' succeeded", "[023-J-12] Deleted $rega host '$h' OK");
    }

    $ua2->get_ok("http://localhost/host/action/?hostop=Delete&hostname=$h", "Delete $regb host $h");
    if ($ua2->content =~ $regexp) {
	$ua2->content_contains("delete '$h' failed.", "[023-J-12] Deleted $regb host '$h' not done, in use");
    } else {
	$ua2->content_contains("delete '$h' succeeded", "[023-J-12] Deleted $regb host '$h' OK");
    }
}

foreach my $ct (@rreg) {
    $ua1->get_ok("http://localhost/contact/action?contactop=Delete&srid=$ct", "Cleanup, delete reclaimed org. object '$ct'...");
    #$ua1->content_contains("delete '$ct' succeeded", "Delete reclaimed org. object '$ct' ok");

    $regexp = qr/(delete '$ct' failed.)/;
    if ($ua1->content =~ $regexp) {
	$ua1->content_contains("delete '$ct' failed.", "Delete reclaimed org. object '$ct' not done, in use");
    } else {
	$ua1->content_contains("delete '$ct' succeeded", "Delete reclaimed org. object '$ct' done");
    }
}

if ($ua0) {
    my %rhd;
    foreach my $ct (@r0admin, @r0tech) {
	next if $rhd{$ct};   # Skip if duplicate IDs
	$ua0->get_ok("http://localhost/contact/action?contactop=Delete&srid=$ct", "Cleanup, delete reg0 contact '$ct'...");
	$ua0->content_contains("delete '$ct' succeeded", "Delete reg0 contact '$ct' OK");
	$rhd{$ct} = 1;
    }
    foreach my $ct (@r0reg) {
	$ua0->get_ok("http://localhost/contact/action?contactop=Delete&srid=$ct", "Cleanup, delete reg0 org. object '$ct'...");
	
	$regexp = qr/(delete '$ct' failed.)/;
	if ($ua0->content =~ $regexp) {
	    $ua0->content_contains("delete '$ct' failed.", "Delete reg0 org. object '$ct' not done, in use");
	} else {
	    $ua0->content_contains("delete '$ct' succeeded", "Delete reg0 org. object '$ct' done");
	}
    }

    foreach my $h (keys %r0) {
	$ua0->get_ok("http://localhost/host/action?hostop=Delete&hostname=$h", "Cleanup, delete reg0 host '$h'...");
	$ua0->content_contains("delete '$h' succeeded", "Delete reg0 host '$h' OK");
    }
}
