#!/usr/bin/perl
use strict;
use warnings;
use Test::More "no_plan";
use Test::WWW::Mechanize::Catalyst 'EppClient';
use URI::Escape;
use HTML::Entities;
use EppClient::Controller::Util;
use EppClient::InitTests;
use YAML;

my $tc = EppClient::InitTests::get_test_config;
my $t = $tc->{t};

my $rega = $t->{rega};
my $pwa  = $t->{pwa};

my $reg0 = $t->{reg0};
my $pw0  = $t->{pw0};

my $cf = EppClient->config();

my $today  = EppClient::Controller::Util::timestamp_as_date(time);
my $future = EppClient::Controller::Util::timestamp_as_date(time + (60*60*48));
my $past   = EppClient::Controller::Util::timestamp_as_date(time - (60*60*48));

# string to make names unique, todays date + process id
my $sfx = "$today-$$";

my $ua1 = Test::WWW::Mechanize::Catalyst->new;

my $ua0;
if ($reg0) {
    $ua0 = Test::WWW::Mechanize::Catalyst->new;
    is(1, 1, "Running with reg0 tests");
} else {
    is(1, 1, "Running without reg0 tests");
}

my (@allhosts, @hosts, @adminc, @techc, $srid_o, $srid_p, @doms, @doms_dnssec);

$ua1->get_ok("http://localhost/", 'get localhost OK');
is($ua1->ct, "text/html", 'text/html OK');
$ua1->title_is("Norid EPP Client", "On the EPP Client's root page");
$ua1->content_contains("You need to log in to use this application", "Yup, on the login page");

### login with correct pw
$ua1->get_ok("http://localhost/login?command=Login&clid=$rega&pw=$pwa", "Login $rega");
$ua1->content_contains("Main menu", "Main menu found - login $rega ok");

if ($ua0) {
    $ua0->get_ok("http://localhost/login?command=Login&clid=$reg0&pw=$pw0", 'Login reg0');
    $ua0->content_contains("Main menu", "Main menu found - login reg0 ok");
}

foreach my $i (1..8) {
    ### Create 8 hosts
    my $hostname="ns$i.bulk-$sfx-$rega.no";
    my $v4_1="1.1.1.$i";
    $ua1->get_ok("http://localhost/host/action/?hostop=Create&hostname=$hostname&v46=$v4_1", "Create host $i: $hostname..." );
    $ua1->content_contains("create '$hostname' succeeded", "Created '$hostname' OK");
    push @allhosts, $hostname;
}
# Use the first 4 hosts for normal use
push @hosts, $allhosts[0], $allhosts[1], $allhosts[2], $allhosts[3];


#########################################################
### Create a registrant object 
#my $contactname_1="Carl Bulks & Sons-$sfx";
my $orgno = $t->{orgnoa3};
my $contactname_1 = $t->{orgnamea3};
my $econtactname_1=encode_entities($contactname_1);

my $type_1='organization';
my $identitytype_1='organizationNumber';

# Need a unique and not yet used orgno for a Brreg-org create
# If in use and quota problems, delete testdomains in this number, or lookup in Brreg and select another number
# which has no domain subscriptions

my $identityvalue_1 = sprintf("%09d", $orgno);
my $cc_1='NO';
my $pc_1='NO-7004';
my $city_1='Trondheim';
my $email_1='bulks@data.no';
my $lcname_1 = $t->{legalcname1};

my $regexp = qr/(Contact with identity \[$identitytype_1:$identityvalue_1\] already exists as \[(\w+)\])/;

if ($cf->{client}->{new_data_model}) {

    $ua1->get_ok("http://localhost/contact/action?contactop=Create&contactorg=".uri_escape($contactname_1)."&contactname=$lcname_1&type=$type_1&cc=$cc_1&pc=$pc_1&city=$city_1&email=$email_1&identitytype=$identitytype_1&identityvalue=$identityvalue_1", "Create Organization...");

    $srid_o = $ua1->form_number('1')->value('srid');
    $ua1->content_contains("create '$lcname_1' succeeded", "Created '$contactname_1' ($srid_o) of type $type_1 OK");
    
} else {

    $ua1->get_ok("http://localhost/contact/action?contactop=Create&contactname=".uri_escape($contactname_1)."&type=$type_1&cc=$cc_1&pc=$pc_1&city=$city_1&email=$email_1&identitytype=$identitytype_1&identityvalue=$identityvalue_1", "Create Organization...");
    
    $srid_o = $ua1->form_number('1')->value('srid');
    # Create regexp to match: Contact with identity [organizationNumber:123456888] already exists as [CBS33O]

    if ($ua1->content =~ $regexp) {
	#print "ok $1\n";
	$srid_o = $2;
	ok($1, "Contact with identity [$identitytype_1:$identityvalue_1] already exists as [$srid_o]");
    } else {   
	$ua1->content_contains("create '$econtactname_1' succeeded", "Created '$contactname_1' ($srid_o) of type $type_1 OK");
    }
}

#print "srid_o: $srid_o\n";

###########################################
# create 4 admin and 4 tech contacts, all affiliated with the org. above

my $affilo="";
if ($cf->{client}->{old_data_model}) {
    $affilo="&organization=$srid_o";
}
    
foreach my $ct ('admin', 'tech') {
    foreach my $i (1..4) {
	my $contactname="Carl Bulks-$i-$ct-$sfx";
	my $type='role';
	if ($cf->{client}->{old_data_model}) {
	    $type='person';
	}
	my $cc='NO';
	my $pc='NO-7004';
	my $city='Trondheim';
	my $email='carl.bulks@data.no';
	
	$ua1->get_ok("http://localhost/contact/action?contactop=Create&contactname=$contactname&type=$type&cc=$cc&pc=$pc&city=$city&email=$email$affilo", "Create Person...");
	my $srid_p = $ua1->form_number('1')->value('srid');

	$ua1->content_contains("create '$contactname' succeeded", "Created '$contactname' ($srid_p) of type $type OK");

	if ($ct eq 'admin') {
	    push @adminc, $srid_p;
	} else {
	    push @techc, $srid_p;
	}
	print "making contact no: $i, type: $ct, srid_p: $srid_p\n";
    }    
}

######
# Create 4 domains
my $nsset;

# Applicant dataset arguments.
my $versionnumber = "2.0";
my $acceptdate = $past . "T14:30:48Z";
my $acceptname = "Tante Sofie";
my $appldataset_args = "versionnumber=$versionnumber&acceptdate=$acceptdate&acceptname=$acceptname&vfyapplicantdatasetupdate=on&objdetails=on";

map {$nsset .= "&nsset=$_" } @hosts;

my $admin="";
if ($cf->{client}->{old_data_model}) {
    map {$admin .= "&admin=$_" } @adminc;
}

my $tech;
map {$tech .= "&tech=$_" } @techc;

#print "nsset: ", $nsset, "\n";
#print "admin: ", $admin, "\n";
#print "tech: ", $tech, "\n";

my $pw = "MySecret";
foreach my $i (1..4) {
    my $domainname = "eppclient-bulktest$i-$sfx-$rega.no";

    $ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=$domainname$nsset&registrant=$srid_o$admin$tech&pw=$pw&$appldataset_args", "Create Domain...");
    unless ($ua1->content_contains("create '$domainname' succeeded", "Create $domainname for test OK")) {
	#print "FAILED: \n", $ua1->content, "\n";
	next;
    }
    push @doms, $domainname;
}



###
#
ok(@doms == 4, "Created 4 test domains"); 

####
# Now we can start some domain bulk testing on the 4 normal domains
#

##############################
# Bulk change the name servers (SetNs)
#
my $bdoms  = join " ", @doms;
my $bnsset = $hosts[1] . " " . $hosts[3];


###############
# ILLEGAL parameters

### missing domain names SetXXX
#
$ua1->get_ok("http://localhost/domainbulk/action?bdomainop=SetNS&bdomainnames=&bnsset=ns1.what.no&bnsset=ns2.what.no", "Domain bulk update: No domain names specified ...");
$ua1->content_contains("'setns' requires at least one domain name", "[023-U-1] At least one domain name required, OK");

$ua1->get_ok("http://localhost/domainbulk/action?bdomainop=SetAdmin&bdomainnames=&bnsset=ns1.what.no&bnsset=ns2.what.no", "Domain bulk update: No domain names specified ...");
$ua1->content_contains("'setadmin' requires at least one domain name", "[023-U-1] At least one domain name required, OK");

$ua1->get_ok("http://localhost/domainbulk/action?bdomainop=SetTech&bdomainnames=&bnsset=ns1.what.no&bnsset=ns2.what.no", "Domain bulk update: No domain names specified ...");
$ua1->content_contains("'settech' requires at least one domain name", "[023-U-1] At least one domain name required, OK");

$ua1->get_ok("http://localhost/domainbulk/action?bdomainop=SetAll&bdomainnames=&bnsset=ns1.what.no&bnsset=ns2.what.no", "Domain bulk update: No domain names specified ...");
$ua1->content_contains("'setall' requires at least one domain name", "[023-U-1] At least one domain name required, OK");


### syntax error in domain SetNs
#
my $illdom = "illegal_dom.no";
$ua1->get_ok("http://localhost/domainbulk/action?bdomainop=SetNS&bdomainnames=$illdom&bnsset=ns1.what.no&bnsset=ns2.what.no", "Domain bulk update: Illegal domain name specified ...");
$ua1->content_contains("setns: illegal domainname: '$illdom'", "[023-U-1]  Illegal domain name specified");



my $onedom = $doms[0];

## SetNs

# Syntax error in name server
my $illns = "ns1.illegal_ns.no";
$ua1->get_ok("http://localhost/domainbulk/action?bdomainop=SetNS&bdomainnames=$onedom&bnsset=$illns", "Domain bulk update: SetNs, illegal name servers specifed ...");
$ua1->content_contains("setns: illegal host name: '$illns'", "[023-U-1] Illegal name server specified");


# missing name servers in SetNS
$ua1->get_ok("http://localhost/domainbulk/action?bdomainop=SetNS&bdomainnames=$bdoms&bnsset=", "Domain bulk update: SetNs, no name servers specifed ...");
$ua1->content_contains("setns: SetNs or SetAll, but no name servers value(s) specified", "[023-U-1] Some name servers must be specified, OK");

# only one domain and one name servers in SetNS
my $onens  = $hosts[0];
$ua1->get_ok("http://localhost/domainbulk/action?bdomainop=SetNS&bdomainnames=$onedom&bnsset=$onens", "Domain bulk update: SetNs, one name servers specifed ...");

#print $ua1->content;

$ua1->content_contains("update: minimum 2 name servers are required for .no", "[023-U-1] Minimum 2 name servers must be specified, OK");

# one domain and max name servers (8) in SetNS
my $allns  = join " ", @allhosts;
$ua1->get_ok("http://localhost/domainbulk/action?bdomainop=SetNS&bdomainnames=$onedom&bnsset=$allns", "Domain bulk update: SetNs, max name servers specifed ...");
$ua1->content_contains("update '$onedom' succeeded", "[023-U-1] SetNs, max name servers specifed");

# one domain and more than max name servers (9) in SetNS
$allns .= " ns1.onemore-$sfx.no";
$ua1->get_ok("http://localhost/domainbulk/action?bdomainop=SetNS&bdomainnames=$onedom&bnsset=$allns", "Domain bulk update: SetNs, more than max name servers specifed ...");
$ua1->content_contains("update '$onedom' failed", "[023-U-1] SetNs, more than max name servers specifed");




### SetAdmin
# illegal admin contact in SetAdmin
my $illct = "ILL12";
my $nxctp = "ILL0012P";
my $nxctr = "ILL0012R";

if ($cf->{client}->{old_data_model}) {
    # illegal admin contact in SetAdmin
    $ua1->get_ok("http://localhost/domainbulk/action?bdomainop=SetAdmin&bdomainnames=$bdoms&badmin=$illct", "Domain bulk update: SetAdmin, illegal admin contacts specifed ...");
    $ua1->content_contains("'setadmin': Invalid Admin contact ID: $illct", "[023-U-2] Illegal admin contact(s) specified");

    # non existing contact in SetAdmin
    $ua1->get_ok("http://localhost/domainbulk/action?bdomainop=SetAdmin&bdomainnames=$bdoms&badmin=$nxctp", "Domain bulk update: SetAdmin, non-existing admin contact specifed ...");
    $ua1->content_contains("Contact with handle [$nxctp] not found.", "[023-U-3] Non-existing admin contact(s) specified");

    unless ($cf->{client}->{new_data_model}) {
	# missing admin contact in SetAdmin
	$ua1->get_ok("http://localhost/domainbulk/action?bdomainop=SetAdmin&bdomainnames=$bdoms&badmin=", "Domain bulk update: SetAdmin, no admin contacts specifed ...");
	$ua1->content_contains("setadmin: SetAdmin or SetAll, but no Admin contact(s) specified", "[023-U-2] Admin contact(s) must be specified, OK");
    }
}

### SetTech

# illegal tech contact in SetTech
$ua1->get_ok("http://localhost/domainbulk/action?bdomainop=SetTech&bdomainnames=$bdoms&btech=$illct", "Domain bulk update: SetTech, illegal tech contacts specifed ...");
$ua1->content_contains("'settech': Invalid Tech contact ID: $illct", "[023-U-3] Illegal tech contact(s) specified");

# non existing tech contact in SetTech
$ua1->get_ok("http://localhost/domainbulk/action?bdomainop=SetTech&bdomainnames=$bdoms&btech=$nxctr", "Domain bulk update: SetTech, non-existing tech contacts specifed ...");

$ua1->content_contains("Contact with handle [$nxctr] not found.", "[023-U-3] Non-existing tech contact(s) specified");

# missing tech contact in SetTech
$ua1->get_ok("http://localhost/domainbulk/action?bdomainop=SetTech&bdomainnames=$bdoms&btech=", "Domain bulk update: SetTech, no tech contacts specifed ...");
$ua1->content_contains("settech: SetTech or SetAll, but no Tech contact(s) specified", "[023-U-3] Tech contact(s) must be specified, OK");


### SetAall
# missing all of ns, admin and tech in SetAll
$ua1->get_ok("http://localhost/domainbulk/action?bdomainop=SetAll&bdomainnames=$bdoms&bnsset=&badmin=&btech=", "Domain bulk update: SetAll, no data specified specifed ...");
$ua1->content_contains("setall: SetNs or SetAll, but no name servers value(s) specified", "[023-U-4] Name servers must be specified, OK");
unless ($cf->{client}->{new_data_model}) {
    $ua1->content_contains("setall: SetAdmin or SetAll, but no Admin contact(s) specified", "[023-U-4] Admin contact(s) must be specified, OK");
}
$ua1->content_contains("setall: SetTech or SetAll, but no Tech contact(s) specified", "[023-U-4] Tech contact(s) must be specified, OK");


### no such domain SetXXX
#
my $nodom = "no-such-domain-$sfx-guaranteed.no";
my $tdom = $nodom . " " . $bdoms;
$ua1->get_ok("http://localhost/domainbulk/action?bdomainop=SetNS&bdomainnames=$tdom&bnsset=$bnsset", "Domain bulk update: No such domain name ...");
$ua1->content_contains("Domain [$nodom] is not delegated.", "[023-U-1] No such domain name delegated");


###################
# LEGAL parameters

# OK params, doit
$ua1->get_ok("http://localhost/domainbulk/action?bdomainop=SetNS&bdomainnames=$bdoms&bnsset=$bnsset", "Domain bulk update: SetNs...");
$ua1->content_contains("In total 4 bulk domains processed, final result:", "[023-U-1] Bulk SetNS on the 4 domains OK");

# Bulk change the adminc (SetAdmin)
my $badmin = "";
if ($cf->{client}->{old_data_model}) {
    $badmin=$adminc[1] . " " . $adminc[3];

    $ua1->get_ok("http://localhost/domainbulk/action?bdomainop=SetAdmin&bdomainnames=$bdoms&badmin=$badmin", "Domain bulk update: SetAdmin...");
    $ua1->content_contains("In total 4 bulk domains processed, final result:", "[023-U-2] Bulk SetAdmin on the 4 domains OK");
}

# Bulk change the tech (SetTech)
my $btech = $techc[1] . " " . $techc[3];
$ua1->get_ok("http://localhost/domainbulk/action?bdomainop=SetTech&bdomainnames=$bdoms&btech=$btech", "Domain bulk update: SetTech...");
$ua1->content_contains("In total 4 bulk domains processed, final result:", "[023-U-3] Bulk SetTech on the 4 domains OK");

# Bulk change all the changable parameters (SetAll)
$bnsset = $hosts[0] . " " . $hosts[2];
$badmin="";
if ($cf->{client}->{old_data_model}) {
    $badmin = "&badmin=" . $adminc[0] . " " . $adminc[2];
}
$btech  = $techc[0] . " " . $techc[2];
$ua1->get_ok("http://localhost/domainbulk/action?bdomainop=SetAll&bdomainnames=$bdoms&bnsset=$bnsset$badmin&btech=$btech", "Domain bulk update: SetAll...");
$ua1->content_contains("In total 4 bulk domains processed, final result:", "[023-U-4] Bulk SetAll on the 4 domains OK");

##
# Bulk renew the 4 domains
# The current expiry is read and set automatically, so just set the duration in months
# 
my $bduration = 6;  # months

# verify off
$ua1->get_ok("http://localhost/domainbulk/action?bdomainop=Renew&bdomainnames=$bdoms&bvfyrenew&bduration=$bduration", "Domain bulk Renew, no verify");
$ua1->content_contains("Renew requires a checked 'verify' field", "[023-U-5] Bulk Renew not verified");

# ok renew
$ua1->get_ok("http://localhost/domainbulk/action?bdomainop=Renew&bdomainnames=$bdoms&bvfyrenew=on&bduration=$bduration", "Domain bulk Renew");
$ua1->content_contains("In total 4 bulk domains processed, final result:", "[023-U-5] Bulk Renew on the 4 domains OK");



##
# Bulk withdraw the 4 domains
# 
# verify off
$ua1->get_ok("http://localhost/domainbulk/action?bdomainop=Withdraw&bdomainnames=$bdoms&bvfywithdraw=", "Domain bulk Withdraw, no verify");
$ua1->content_contains("Withdraw requires a checked 'verify' field", "[023-U-6] Bulk Withdraw not verified");

# ok withdraw
$ua1->get_ok("http://localhost/domainbulk/action?bdomainop=Withdraw&bdomainnames=$bdoms&bvfywithdraw=on", "Domain bulk Withdraw");
$ua1->content_contains("In total 4 bulk domains processed, final result:", "[023-U-6] Bulk Withdraw on the 4 domains OK");


###########################################
# Bulk transfer the 4 domains from reg0.

# Collect all the contactIDs on the reg0 account
my (@rreg, @radmin, @rtech, @r0reg, @r0admin, @r0tech, @r0nsset);

my %r0;
if ($ua0) {
    foreach my $dom (@doms) {
	$ua0->get_ok("http://localhost/domain/action?domainop=Info&domainname=$dom", "Info on reg0 domain $dom ...");
	$ua0->content_contains("info '$dom' succeeded", "Info on reg0 domain $dom succeeded");
	
	push @r0reg  , $ua0->form_number('1')->value('registrant');
	push @r0admin, $ua0->form_number('1')->param('admin');
	push @r0tech , $ua0->form_number('1')->param('tech');
	
	@r0nsset = $ua0->form_number('1')->param('nsset');
	foreach my $n (@r0nsset) {
	    $r0{$n} = 1;
	} 
    }
}

# Bulk transfer not verified
$ua1->get_ok("http://localhost/domainbulk/action?bdomainop=Transfer_wAuthInfo&bdomainnames=$bdoms&bvfytransferwauthinfo=", "Transfer domain bulk");
$ua1->content_contains("Transfer with authInfo requires a checked 'verify' field", "[023-K-2, 023-K-3] Bulk transfer with AuthInfo rejected - not verified");

# Bulk transfer verified
$ua1->get_ok("http://localhost/domainbulk/action?bdomainop=Transfer_wAuthInfo&bdomainnames=$bdoms&bvfytransferwauthinfo=on", "Transfer domain bulk");

foreach my $dom (@doms) {
    $ua1->content_contains("transfer_execute '$dom' succeeded.", "[023-K-2, 023-K-3] Bulk transfer $dom with AuthInfo OK");
    $ua1->content_contains("Transfer status is 'serverApproved'.", "[023-K-2, 023-K-3] Bulk transfer $dom approved");
}

# Collect all the contactIDs after reclaim
foreach my $dom (@doms) {
    $ua1->get_ok("http://localhost/domain/action?domainop=Info&domainname=$dom", "Info on $rega domain $dom ...");
    $ua1->content_contains("info '$dom' succeeded", "Info on $rega domain $dom succeeded");

    # Collect all the contactIDs after reclaim
    push @rreg  , $ua1->form_number('1')->value('registrant');
    push @radmin, $ua1->form_number('1')->param('admin');
    push @rtech , $ua1->form_number('1')->param('tech');
}

#print $ua1->content;
#print "before reclaim, r0reg: @r0reg, r0admin: @r0admin, r0tech: @r0tech\n";
#print "after reclaim, rreg: @rreg, radmin: @radmin, rtech: @rtech\n";


####
# Bulk set authInfo to some value

my $bpassword="MyBulkZecret123";
$ua1->get_ok("http://localhost/domainbulk/action?bdomainop=SetAuthInfo&bdomainnames=$bdoms&bpw=$bpassword", "Bulk set AuthInfo");

foreach my $dom (@doms) {
    $ua1->content_contains("update '$dom' succeeded.", "[023-K-2, 023-K-3] Bulk set $dom AuthInfo OK");
}


#########
#
# DNSSEC domain bulk operations.
# Only remove all DNSSEC data is currently supported, and it need a verify flag.
#

SKIP: { # dnssec

    unless ($cf->{dnssec}->{enabled}) {
	skip "DNSSEC is disabled - all DNSSEC tests skipped";
    }

##
# Initialize: Create 4 specific test domains with DNSSEC data
#
my $keytag1    = 9286;
my $keytag2    = 22780;

my $alg        = 8;

my $digtype1   = 1;
my $digtype2   = 2;

my $digest1_1  = lc('A970213D5E33F0952DEF8C849DEADC6FFC7E3317');
my $digest1_2  = lc('92217DCCAF8C50EC0F8DF62B821A3A506ED9163A589D839FEA4119D02A6C5EEA');

my $dnssec_ds1_1  = "keyTag=$keytag1&alg=$alg&digestType=$digtype1&digest=$digest1_1";
my $dnssec_ds1_2  = "keyTag=$keytag1&alg=$alg&digestType=$digtype2&digest=$digest1_2";

my $dnssec_ds_args1   = "$dnssec_ds1_1&$dnssec_ds1_2";

foreach my $i (1..4) {
    my $domainname = "eppclient-bulktest-dnssec$i-$sfx-$rega.no";

    $ua1->get_ok("http://localhost/domain/action?domainop=Create&domainname=$domainname$nsset&registrant=$srid_o$admin$tech&pw=$pw&$appldataset_args&$dnssec_ds_args1", "Create DNSSEC Domain...");
    unless ($ua1->content_contains("create '$domainname' succeeded", "Create DNSSEC $domainname for test OK")) {
	#print "FAILED: \n", $ua1->content, "\n";
	next;
    }
    push @doms_dnssec, $domainname;
}

ok(@doms_dnssec == 4, "Created 4 test domains with DNSSEC"); 

##
# Remove all DNSSEC data in bulk for the 4 domains
my $bdoms_dnssec  = join " ", @doms_dnssec;
$ua1->get_ok("http://localhost/domainbulk/action?bdomainop=RemoveDNSSEC&bdomainnames=$bdoms_dnssec", "Domain bulk DNSSEC data remove, not verified ...");
$ua1->content_contains("Removal of all DNSSEC data requires a checked 'verify' field", "[023-U-8] Bulk update and remove all DNSSEC data - not verified");
$ua1->get_ok("http://localhost/domainbulk/action?bdomainop=RemoveDNSSEC&bdomainnames=$bdoms_dnssec&bvfyremalldnssec=on", "Domain bulk DNSSEC data removal, verified ...");

foreach my $d (@doms_dnssec) {
    $ua1->content_contains("info '$d' succeeded",   "[023-U-8] info on $d ok");
    $ua1->content_contains("update '$d' succeeded", "[023-U-8] update and removal of all DNSSEC data info on $d ok");
}
$ua1->content_contains("In total 4 bulk domains processed, final result:", "[023-U-8] Bulk removal of DNSSEC data on 4 domains succeeded");

##
# cleanup: bulk delete the 4 dnssec domains
$ua1->get_ok("http://localhost/domainbulk/action?bdomainop=Delete&bdomainnames=$bdoms_dnssec&bvfydelete=on", "Domain bulk delete of the DNSSEC domains..");
$ua1->content_contains("In total 4 bulk domains processed, final result:", "[023-U-7] Bulk Delete without dates on 4 domains");

} # end skip dnssec


####
# Finally, bulk delete the 4 normal domains

# verify off
$ua1->get_ok("http://localhost/domainbulk/action?bdomainop=Delete&bdomainnames=$bdoms&bvfydelete", "Domain bulk Delete, no verify");
$ua1->content_contains("Delete requires a checked 'verify' field", "[023-U-7] Bulk Delete not verified");

# ok delete

# delete two of the domains without dates
my $bdoms0_1  = $doms[0] . " " . $doms[1];
$ua1->get_ok("http://localhost/domainbulk/action?bdomainop=Delete&bdomainnames=$bdoms0_1&bvfydelete=on", "Domain bulk Delete, without dates");
$ua1->content_contains("In total 2 bulk domains processed, final result:", "[023-U-7] Bulk Delete without dates on 2 domains");

# delete the two remaing domains with dates
my $now = time;
my $d1 = EppClient::Controller::Util::timestamp_as_date($now+3600);
my $d2 = EppClient::Controller::Util::timestamp_as_date($now+3600+3600);
my $bdoms2_3  = $doms[2] . " " . $doms[3];
$ua1->get_ok("http://localhost/domainbulk/action?bdomainop=Delete&bdomainnames=$bdoms2_3&bvfydelete=on&bdeletefromdns=$d1&bdeletefromregistry=$d2", "Domain bulk Delete, with dates");
$ua1->content_contains("In total 2 bulk domains processed, final result:", "[023-U-7] Bulk Delete with dates on 2 domains");


##
# Other cleanup. Delete the Contacts and hosts created above

# ua1 deletes
foreach my $ct (@adminc, @techc, @radmin, @rtech) {
    $ua1->get_ok("http://localhost/contact/action?contactop=Delete&srid=$ct", "Cleanup, delete contact '$ct'...");

    $regexp = qr/(delete '$ct' failed.)/;
    if ($ua1->content =~ $regexp) {
	$ua1->content_contains("Object [$ct] is in use and cannot be deleted.", "Delete '$ct' contact not done, in use");
    } else {
	$ua1->content_contains("delete '$ct' succeeded", "Delete '$ct' contact OK");
    }
}

my %orgs;
foreach my $ct ($srid_o, @rreg) {
    $orgs{$ct} = 1;
}
foreach my $ct (keys %orgs) {
    $ua1->get_ok("http://localhost/contact/action?contactop=Delete&srid=$ct", "Cleanup, delete org. object '$ct'...");

    $regexp = qr/(delete '$ct' failed.)/;
    if ($ua1->content =~ $regexp) {
	$ua1->content_contains("delete '$ct' failed.", "Delete '$ct' contact not done, in use");
    } else {
	$ua1->content_contains("delete '$ct' succeeded", "Delete '$ct' contact done");
    }
}

if ($ua0) {
    # ua0 deletes
    foreach my $ct (@r0admin, @r0tech) {
	$ua0->get_ok("http://localhost/contact/action?contactop=Delete&srid=$ct", "Cleanup, delete contact '$ct'...");
	
	$regexp = qr/(delete '$ct' failed.)/;
	if ($ua0->content =~ $regexp) {
	    $ua0->content_contains("delete '$ct' failed.", "Delete '$ct' contact not done, in use");
	} else {
	    $ua0->content_contains("delete '$ct' succeeded", "Delete contact '$ct' OK");
	}
    }

    foreach my $ct (@r0reg) {
	$ua0->get_ok("http://localhost/contact/action?contactop=Delete&srid=$ct", "Cleanup, delete org. object '$ct'...");
	
	$regexp = qr/(delete '$ct' failed.)/;
	if ($ua0->content =~ $regexp) {
	    $ua0->content_contains("delete '$ct' failed.", "Delete '$ct' contact not done, in use");
	} else {
	    $ua0->content_contains("delete '$ct' succeeded", "Delete '$ct' contact done");
	}
    }
}

foreach my $h (@allhosts) {
    $ua1->get_ok("http://localhost/host/action?hostop=Delete&hostname=$h", "Cleanup, delete $rega host '$h'...");

    $regexp = qr/(delete '$h' failed.)/;
    if ($ua1->content =~ $regexp) {
	$ua1->content_contains("delete '$h' failed.", "[023-J-12] Deleted host '$h' not done, in use");
    } else {
	$ua1->content_contains("delete '$h' succeeded", "[023-J-12] Deleted host '$h' OK");
    }

}
if ($ua0) {
    foreach my $h (keys %r0) {
	$ua0->get_ok("http://localhost/host/action?hostop=Delete&hostname=$h", "Cleanup, delete reg0 host '$h'...");
	$ua0->content_contains("delete '$h' succeeded", "Delete host '$h' OK");
    }
}
