#!/usr/bin/perl
use strict;
use warnings;
use Test::More tests => 13;
use Test::WWW::Mechanize::Catalyst 'EppClient';
use EppClient::InitTests;

my $tc = EppClient::InitTests::get_test_config;
my $t = $tc->{t};

my $rega = $t->{rega};
my $regb = $t->{regb};
my $pwa  = $t->{pwa};
my $pwb  = $t->{pwb};

my $ua1 = Test::WWW::Mechanize::Catalyst->new;
my $ub1 = Test::WWW::Mechanize::Catalyst->new;

### login with correct pw
$ua1->get_ok("http://localhost/login?command=Login&clid=$rega&pw=$pwa", 'Login OK');
$ub1->get_ok("http://localhost/login?command=Login&clid=$regb&pw=$pwb", 'Login OK');


# Do a message poll operation to produced to data in the log file
$ua1->get_ok("http://localhost/message/action/?messageop=Poll", "Poll message queue to get status and id");


### Open the "My Last EPP Protocol logs" using registrar A
$ua1->get_ok("http://localhost/options/logdir/xml", "As $rega, try to get the protocol log page for current registrar ($rega)");
$ua1->content_contains("The file marked with *) is your current session log file.", "[023-Q-5] Loaded the log page OK");
my $raw_link = $ua1->find_link( url_regex => qr/\/raw\//i );
my $pp_link  = $ua1->find_link( url_regex => qr/\/pp\//i );
my $pps_link  = $ua1->find_link( url_regex => qr/\/pps\//i );
my $raw_link_url = $raw_link->url();
my $pp_link_url = $pp_link->url();
my $pps_link_url = $pps_link->url();


### Open the raw, pp and pps log files
#print "url: " . $raw_link_url . "\n";
$ua1->get_ok($raw_link_url, "Get the protocol raw log for current registrar ($rega)");
#print $ua1->content;
$ua1->content_contains("clTRID", "[023-Q-5] Loaded the raw log OK");

#print "url: " . $pp_link_url . "\n";
$ua1->get_ok($pp_link_url, "Get the protocol pp log for current registrar ($rega)");
#print $ua1->content;
$ua1->content_contains("clTRID", "[023-Q-5] Loaded the pp log OK");

#print "url: " . $pps_link_url . "\n";
$ua1->get_ok($pps_link_url, "Get the protocol pps log for current registrar ($rega)");
#print $ua1->content;
$ua1->content_contains("clTRID", "[023-Q-5] Loaded the pps log OK");


### Open the "My Last EPP Client logs" using registrar B
$ub1->get_ok("http://localhost/options/logdir/xml", "As $regb, try to get the protocol log page for registrar $regb");
$ub1->content_contains("The file marked with *) is your current session log file.", "[023-Q-5] Loaded the log page OK");

