#!/usr/bin/perl
use strict;
use warnings;
use Test::More "no_plan";
use Test::WWW::Mechanize::Catalyst 'EppClient';
use EppClient::InitTests;

my $tc = EppClient::InitTests::get_test_config;
my $t = $tc->{t};

my $rega = $t->{rega};
my $pwa  = $t->{pwa};

my $ua1 = Test::WWW::Mechanize::Catalyst->new;
my ($count, $lastid);

### login with correct pw
$ua1->get_ok("http://localhost/login?command=Login&clid=$rega&pw=$pwa", "Login OK");
$ua1->content_contains("Main menu", "Main menu found");

###
# First, some error variants

# poll, get lastid, ack it, then ack the next id without polling first, shall be rejected
$ua1->get_ok("http://localhost/message/action/?messageop=Poll", "Poll message queue to get status and id");

($count, $lastid) = get_last_id($ua1);
if ($count > 0) {
    $ua1->get_ok("http://localhost/message/action/?messageop=Acknowledge&lastid=$lastid&cmddetails=on", "Last id: $lastid polled/fetched, acknowledging it");
    $ua1->content_contains("acknowledge '$lastid' succeeded.","[023-V-2] acknowledge '$lastid' succeeded");
} else {
    ok($count == 0, "[023-V-2] Placeholder1: Count is 0, cannot acknowledge ok just now");
    ok($count == 0, "[023-V-2] Placeholder2: Count is 0, cannot acknowledge ok just now");
}

($count, $lastid) = get_last_id($ua1);
if ($count > 0) {
    $ua1->get_ok("http://localhost/message/action/?messageop=Acknowledge&lastid=$lastid", "Last id: $lastid acknowledging it without polling first");
    $ua1->content_contains("acknowledge '$lastid' failed.","[023-V-2] Last id: $lastid acknowledging it without polling first failed");
    $ua1->content_contains("Message with id [$lastid] has not been polled.","[023-V-2] acknowledge '$lastid' failed - not been polled");
} else {
    ok($count == 0, "[023-V-2] Placeholder1: Count is 0, cannot acknowledge/failed just now");
    ok($count == 0, "[023-V-2] Placeholder2: Count is 0, cannot acknowledge/failed just now");
    ok($count == 0, "[023-V-2] Placeholder3: Count is 0, cannot acknowledge/failed just now");
}

## 
# Now, if there are any more service messages queued, poll them all out
#
my $cnt = 1;
my $poll_ok  = 0;
my $poll_err = 0;
my $ack_ok   = 0;
my $ack_err  = 0;
my $totcount = $count;

#print "totcount; $totcount\n";

while ($lastid && $count > 0 && $cnt > 0) {
    #print "lastid1 loop: $lastid\n";
    $ua1->get_ok("http://localhost/message/action/?messageop=Poll&lastid=$lastid&cmddetails=", "Poll message queue to get status and id. totcount: $totcount, count: $count");
    my $id1 = $lastid;

    ($count, $lastid) = get_last_id($ua1);

    if ($count > 0) {
	#print "lastid2 loop: $lastid\n";
	#print $ua1->content;
	my $regexp = qr/poll '$id1' succeeded/;

	if ($ua1->content =~ $regexp) {
	    ++$poll_ok;
	} else {
	    ++$poll_err;
	}

	$ua1->get_ok("http://localhost/message/action/?messageop=Acknowledge&lastid=$lastid&cmddetails=on", "Last id: $lastid polled/fetched, acknowledging it..");

	$regexp = qr/acknowledge '$lastid' succeeded./;

	if ($ua1->content =~ $regexp) {
	    ++$ack_ok;
	} else {
	    ++$ack_err;
	}
    }
}
ok($poll_ok >= $totcount, "[023-V-1] polled '$totcount' messages succeeded. poll_ok: $poll_ok, poll_err: $poll_err");
ok($poll_err == 0    , "[023-V-1] poll error on 0 messages");

ok($ack_ok >= $totcount, "[023-V-2] acknowledge '$totcount' messages succeeded. ack_ok: $ack_ok, ack_err: $ack_err");
ok($ack_err == 0    , "[023-V-2] acknowledge error on 0 messages");


##
# Now, if no messages remains, do a poll
#
if ($count == 0) {
    $ua1->get_ok("http://localhost/message/action/?messageop=Poll", "Poll an empty message queue to get status");
    $ua1->content_contains("poll  succeeded.", "[023-V-1] poll empty queue succeeded");
    ($count, $lastid) = get_last_id($ua1);

    ok($count == 0, "[023-V-1] Count is 0");
    ok($lastid == 0, "[023-V-1] Last id not set");

    # ack with with nothing to acknowledge
    # set some wild last id number to make it a valid number
    $lastid=22;
    $ua1->get_ok("http://localhost/message/action/?messageop=Acknowledge&lastid=$lastid&cmddetails=", "Acknowledge Last id: $lastid - nothing to acknowledge, but try anyway - shall fail");
    $ua1->content_contains("acknowledge '$lastid' failed.", "[023-V-2] Acknowledge Last id: $lastid - nothing to acknowledge, but try anyway - failed");
}

sub get_last_id {
    my $ua1 = shift;
    
    my $count = 0;
    my $lastid = 0;

    my $regexp = qr/Messages waiting/;
    if ($ua1->content =~ $regexp) {
	# a count shall always be found
	$count = $ua1->form_number('1')->value('count');
    }
    if ($count && $count > 0) {
	# a lastid shall now be found
	$regexp = qr/Last id/;
	if ($ua1->content =~ $regexp) {
	    $lastid = $ua1->form_number('1')->value('lastid');
	}
    }
    return $count, $lastid;
}
